# coding: utf-8

"""
    Weheat Backend

    This is the backend for the Weheat project

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from typing import Any, ClassVar, Dict, List, Optional, Union
from pydantic import BaseModel, StrictBool, StrictFloat, StrictInt, StrictStr
from pydantic import Field
try:
    from typing import Self
except ImportError:
    from typing_extensions import Self

class RawHeatPumpLogDto(BaseModel):
    """
    Raw heat pump log as given by a heat pump  (Required star states that it will always be there for any roll, otherwise it is filtered for customers)
    """ # noqa: E501
    heat_pump_id: StrictStr = Field(description="Identifier of the Heat Pump", alias="heatPumpId")
    timestamp: datetime = Field(description="Timestamp of when this was logged to the server")
    state: Optional[StrictInt] = Field(default=None, description="Current heat pump state (which is decoded in the view)")
    rpm_ctrl: Optional[StrictInt] = Field(default=None, description="Current mode of RPM control (which is decoded in the view)", alias="rpmCtrl")
    error: Optional[StrictInt] = Field(default=None, description="Current state of error in the heat pump (which is decoded in the view)")
    error_decoded_dtc_none: Optional[StrictBool] = Field(default=None, description="None state of the error flag which isn't raw for practical timescale purposes (it was bitwise encoded)", alias="errorDecodedDtcNone")
    error_decoded_dtc_continue: Optional[StrictBool] = Field(default=None, description="DTC continue state of the error flag which isn't raw for practical timescale purposes (it was bitwise encoded)", alias="errorDecodedDtcContinue")
    error_decoded_dtc_compressor_off: Optional[StrictBool] = Field(default=None, description="DTC off state of the error flag which isn't raw for practical timescale purposes (it was bitwise encoded)", alias="errorDecodedDtcCompressorOff")
    error_decoded_dtc_defrost_forbidden: Optional[StrictBool] = Field(default=None, description="DTC defrost forbidden state of the error flag which isn't raw for practical timescale purposes (it was bitwise encoded)", alias="errorDecodedDtcDefrostForbidden")
    error_decoded_dtc_request_service: Optional[StrictBool] = Field(default=None, description="DTC request service state of the error flag which isn't raw for practical timescale purposes (it was bitwise encoded)", alias="errorDecodedDtcRequestService")
    error_decoded_dtc_use_heating_curve: Optional[StrictBool] = Field(default=None, description="DTC use heating curve state of the error flag which isn't raw for practical timescale purposes (it was bitwise encoded)", alias="errorDecodedDtcUseHeatingCurve")
    error_decoded_dtc_dhw_forbidden: Optional[StrictBool] = Field(default=None, description="DTC use heating curve state of the error flag which isn't raw for practical timescale purposes (it was bitwise encoded)", alias="errorDecodedDtcDhwForbidden")
    error_decoded_dtc_error: Optional[StrictBool] = Field(default=None, description="DTC use heating curve state of the error flag which isn't raw for practical timescale purposes (it was bitwise encoded)", alias="errorDecodedDtcError")
    error_decoded_dtc_inactive: Optional[StrictBool] = Field(default=None, description="DTC Inactive state of the error flag which isn't raw for practical timescale purposes (it was bitwise encoded)", alias="errorDecodedDtcInactive")
    control_bridge_status: Optional[StrictInt] = Field(default=None, description="Raw control bridge status of the heat pump (bitwise encoded)", alias="controlBridgeStatus")
    control_bridge_status_decoded_water_pump: Optional[StrictBool] = Field(default=None, description="Water pump (requested) on/off flag of the control bridge status (0x01)", alias="controlBridgeStatusDecodedWaterPump")
    control_bridge_status_decoded_dhw_valve: Optional[StrictBool] = Field(default=None, description="DhwValve (requested) open/closed flag of the control bridge status (0x02) (DHW Only)", alias="controlBridgeStatusDecodedDhwValve")
    control_bridge_status_decoded_gas_boiler: Optional[StrictBool] = Field(default=None, description="Gas boiler assistance (requested) on/off flag of the control bridge status (0x04) (Hybrid Only)", alias="controlBridgeStatusDecodedGasBoiler")
    control_bridge_status_decoded_electric_heater: Optional[StrictBool] = Field(default=None, description="Electric heater assistance (requested) on/off flag of the control bridge status (0x08) (DHW Only)", alias="controlBridgeStatusDecodedElectricHeater")
    control_bridge_status_decoded_water_pump2: Optional[StrictBool] = Field(default=None, description="Second water pump (requested) on/off flag of the control bridge status (0x010) (DHW Only)", alias="controlBridgeStatusDecodedWaterPump2")
    input_status: Optional[StrictInt] = Field(default=None, description="Raw input status of the heat pump", alias="inputStatus")
    current_control_method: Optional[StrictInt] = Field(default=None, description="Raw current control method state of the heat pump", alias="currentControlMethod")
    signal_strength: Optional[StrictInt] = Field(default=None, description="Signal strength (rssi) reported by the modem of the control board", alias="signalStrength")
    t1: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Temperature T1 (TOP) of the DHW sensors")
    t2: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Temperature T2 (BOTTOM) of the DHW sensors")
    t_spare_ntc: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Spare temperature sensor", alias="tSpareNtc")
    t_board: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Temperature of the control board", alias="tBoard")
    t_air_in: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Temperature of the air going into the heat pump", alias="tAirIn")
    t_air_out: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Temperature of the air going out of the heat pump", alias="tAirOut")
    t_water_in: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Temperature of the water going into the heat pump", alias="tWaterIn")
    t_water_out: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Temperature of the water going out of the heat pump", alias="tWaterOut")
    t_water_house_in: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Temperature of the water going into the building", alias="tWaterHouseIn")
    rpm: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Rpm of the compressor")
    rpm_limiter: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Rpm limit of the compressor", alias="rpmLimiter")
    rpm_limiter_type: Optional[StrictInt] = Field(default=None, description="Type of why the rpm is limited", alias="rpmLimiterType")
    p_compressor_in: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Pressure of the compressor of the side where air goes into the compressor", alias="pCompressorIn")
    p_compressor_out: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Pressure of the compressor of the side where air goes out of the compressor", alias="pCompressorOut")
    p_compressor_in_target: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Target pressure of the compressor of the side where air goes into the compressor", alias="pCompressorInTarget")
    t_inverter: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Temperature of the inverter", alias="tInverter")
    t_compressor_in: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Temperature of the compressor of the side where air goes into the compressor", alias="tCompressorIn")
    t_compressor_out: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Temperature of the compressor of the side where air goes out of the compressor", alias="tCompressorOut")
    t_compressor_in_transient: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Temperature transient of the compressor of the side where air goes into the compressor", alias="tCompressorInTransient")
    t_compressor_out_transient: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Temperature transient of the compressor of the side where air goes out of the compressor", alias="tCompressorOutTransient")
    delta_t_compressor_in_superheat: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Difference in temperature for superheat based on the compressor temperatures", alias="deltaTCompressorInSuperheat")
    compressor_power_low_accuracy: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Power used by the compressor (low accuracy)", alias="compressorPowerLowAccuracy")
    fan: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="State of the fan")
    fan_power: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Power used by the fan", alias="fanPower")
    valve: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Position value of the valve")
    cm_mass_power_in: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Power going into the heat pump", alias="cmMassPowerIn")
    cm_mass_power_out: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Power going out of the heat pump (in the form of usable heat)", alias="cmMassPowerOut")
    temperature_error_integral: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Current value of the temperature error integral", alias="temperatureErrorIntegral")
    on_off_thermostat_state: Optional[StrictInt] = Field(default=None, description="State of the on/off thermostat", alias="onOffThermostatState")
    thermostat_status: Optional[StrictInt] = Field(default=None, description="Status of the OpenTherm thermostat", alias="thermostatStatus")
    t_room: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Temperature of the room given by the OpenTherm thermostat", alias="tRoom")
    t_room_target: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Target temperature of the room given by the OpenTherm thermostat", alias="tRoomTarget")
    t_thermostat_setpoint: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Setpoint temperature of the OpenTherm thermostat OR the water setpoint of the heat pump", alias="tThermostatSetpoint")
    ot_boiler_status: Optional[StrictInt] = Field(default=None, description="Status of the OpenTherm boiler", alias="otBoilerStatus")
    ot_boiler_feed_temperature: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Feed temperature of the OpenTherm boiler", alias="otBoilerFeedTemperature")
    ot_boiler_return_temperature: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Return temperature of the OpenTherm boiler", alias="otBoilerReturnTemperature")
    inverter_input_voltage: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Input voltage measured by the compressor inverter  float - 4 bytes", alias="inverterInputVoltage")
    central_heating_pwm_requested_duty_cycle: Optional[StrictInt] = Field(default=None, description="Requested duty cycle to pwm pump for central heating circuit", alias="centralHeatingPwmRequestedDutyCycle")
    central_heating_flow: Optional[StrictInt] = Field(default=None, description="Duty cycle read by pwm pump for central heating circuit", alias="centralHeatingFlow")
    dhw_pwm_requested_duty_cycle: Optional[StrictInt] = Field(default=None, description="Requested duty cycle to pwm pump for DHW circuit", alias="dhwPwmRequestedDutyCycle")
    dhw_flow: Optional[StrictInt] = Field(default=None, description="Duty cycle read by pwm pump for DHW circuit", alias="dhwFlow")
    indoor_unit_heater_temperature: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Temperature of the indoor unit heater", alias="indoorUnitHeaterTemperature")
    indoor_unit_input_current: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Current of the indoor unit heater", alias="indoorUnitInputCurrent")
    sinr: Optional[StrictInt] = Field(default=None, description="Signal to noise ratio of the modem of the control board (SINR)")
    cooling_status: Optional[StrictInt] = Field(default=None, description="Current status of cooling  With enum values: 0 = idle, 1 = starting, 2 = active, 3 = stopping", alias="coolingStatus")
    debug_variable1: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Optional debug variable that can be set to anything in firmware and will be logged as a float  float - 4 bytes", alias="debugVariable1")
    debug_variable2: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Optional? debug variable that can be set to anything in firmware and will be logged as a float  float - 4 bytes", alias="debugVariable2")
    debug_variable3: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Optional debug variable that can be set to anything in firmware and will be logged as a float  float - 4 bytes", alias="debugVariable3")
    debug_variable4: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Optional debug variable that can be set to anything in firmware and will be logged as a float  float - 4 bytes", alias="debugVariable4")
    debug_variable5: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Optional debug variable that can be set to anything in firmware and will be logged as a float  float - 4 bytes", alias="debugVariable5")
    interval: StrictInt = Field(description="Interval for this log in seconds")
    __properties: ClassVar[List[str]] = ["heatPumpId", "timestamp", "state", "rpmCtrl", "error", "errorDecodedDtcNone", "errorDecodedDtcContinue", "errorDecodedDtcCompressorOff", "errorDecodedDtcDefrostForbidden", "errorDecodedDtcRequestService", "errorDecodedDtcUseHeatingCurve", "errorDecodedDtcDhwForbidden", "errorDecodedDtcError", "errorDecodedDtcInactive", "controlBridgeStatus", "controlBridgeStatusDecodedWaterPump", "controlBridgeStatusDecodedDhwValve", "controlBridgeStatusDecodedGasBoiler", "controlBridgeStatusDecodedElectricHeater", "controlBridgeStatusDecodedWaterPump2", "inputStatus", "currentControlMethod", "signalStrength", "t1", "t2", "tSpareNtc", "tBoard", "tAirIn", "tAirOut", "tWaterIn", "tWaterOut", "tWaterHouseIn", "rpm", "rpmLimiter", "rpmLimiterType", "pCompressorIn", "pCompressorOut", "pCompressorInTarget", "tInverter", "tCompressorIn", "tCompressorOut", "tCompressorInTransient", "tCompressorOutTransient", "deltaTCompressorInSuperheat", "compressorPowerLowAccuracy", "fan", "fanPower", "valve", "cmMassPowerIn", "cmMassPowerOut", "temperatureErrorIntegral", "onOffThermostatState", "thermostatStatus", "tRoom", "tRoomTarget", "tThermostatSetpoint", "otBoilerStatus", "otBoilerFeedTemperature", "otBoilerReturnTemperature", "inverterInputVoltage", "centralHeatingPwmRequestedDutyCycle", "centralHeatingFlow", "dhwPwmRequestedDutyCycle", "dhwFlow", "indoorUnitHeaterTemperature", "indoorUnitInputCurrent", "sinr", "coolingStatus", "debugVariable1", "debugVariable2", "debugVariable3", "debugVariable4", "debugVariable5", "interval"]

    model_config = {
        "populate_by_name": True,
        "validate_assignment": True,
        "protected_namespaces": (),
    }


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of RawHeatPumpLogDto from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={
            },
            exclude_none=True,
        )
        # set to None if rpm_ctrl (nullable) is None
        # and model_fields_set contains the field
        if self.rpm_ctrl is None and "rpm_ctrl" in self.model_fields_set:
            _dict['rpmCtrl'] = None

        # set to None if error (nullable) is None
        # and model_fields_set contains the field
        if self.error is None and "error" in self.model_fields_set:
            _dict['error'] = None

        # set to None if error_decoded_dtc_none (nullable) is None
        # and model_fields_set contains the field
        if self.error_decoded_dtc_none is None and "error_decoded_dtc_none" in self.model_fields_set:
            _dict['errorDecodedDtcNone'] = None

        # set to None if error_decoded_dtc_continue (nullable) is None
        # and model_fields_set contains the field
        if self.error_decoded_dtc_continue is None and "error_decoded_dtc_continue" in self.model_fields_set:
            _dict['errorDecodedDtcContinue'] = None

        # set to None if error_decoded_dtc_compressor_off (nullable) is None
        # and model_fields_set contains the field
        if self.error_decoded_dtc_compressor_off is None and "error_decoded_dtc_compressor_off" in self.model_fields_set:
            _dict['errorDecodedDtcCompressorOff'] = None

        # set to None if error_decoded_dtc_defrost_forbidden (nullable) is None
        # and model_fields_set contains the field
        if self.error_decoded_dtc_defrost_forbidden is None and "error_decoded_dtc_defrost_forbidden" in self.model_fields_set:
            _dict['errorDecodedDtcDefrostForbidden'] = None

        # set to None if error_decoded_dtc_request_service (nullable) is None
        # and model_fields_set contains the field
        if self.error_decoded_dtc_request_service is None and "error_decoded_dtc_request_service" in self.model_fields_set:
            _dict['errorDecodedDtcRequestService'] = None

        # set to None if error_decoded_dtc_use_heating_curve (nullable) is None
        # and model_fields_set contains the field
        if self.error_decoded_dtc_use_heating_curve is None and "error_decoded_dtc_use_heating_curve" in self.model_fields_set:
            _dict['errorDecodedDtcUseHeatingCurve'] = None

        # set to None if error_decoded_dtc_dhw_forbidden (nullable) is None
        # and model_fields_set contains the field
        if self.error_decoded_dtc_dhw_forbidden is None and "error_decoded_dtc_dhw_forbidden" in self.model_fields_set:
            _dict['errorDecodedDtcDhwForbidden'] = None

        # set to None if error_decoded_dtc_error (nullable) is None
        # and model_fields_set contains the field
        if self.error_decoded_dtc_error is None and "error_decoded_dtc_error" in self.model_fields_set:
            _dict['errorDecodedDtcError'] = None

        # set to None if error_decoded_dtc_inactive (nullable) is None
        # and model_fields_set contains the field
        if self.error_decoded_dtc_inactive is None and "error_decoded_dtc_inactive" in self.model_fields_set:
            _dict['errorDecodedDtcInactive'] = None

        # set to None if control_bridge_status (nullable) is None
        # and model_fields_set contains the field
        if self.control_bridge_status is None and "control_bridge_status" in self.model_fields_set:
            _dict['controlBridgeStatus'] = None

        # set to None if control_bridge_status_decoded_water_pump (nullable) is None
        # and model_fields_set contains the field
        if self.control_bridge_status_decoded_water_pump is None and "control_bridge_status_decoded_water_pump" in self.model_fields_set:
            _dict['controlBridgeStatusDecodedWaterPump'] = None

        # set to None if control_bridge_status_decoded_dhw_valve (nullable) is None
        # and model_fields_set contains the field
        if self.control_bridge_status_decoded_dhw_valve is None and "control_bridge_status_decoded_dhw_valve" in self.model_fields_set:
            _dict['controlBridgeStatusDecodedDhwValve'] = None

        # set to None if control_bridge_status_decoded_gas_boiler (nullable) is None
        # and model_fields_set contains the field
        if self.control_bridge_status_decoded_gas_boiler is None and "control_bridge_status_decoded_gas_boiler" in self.model_fields_set:
            _dict['controlBridgeStatusDecodedGasBoiler'] = None

        # set to None if control_bridge_status_decoded_electric_heater (nullable) is None
        # and model_fields_set contains the field
        if self.control_bridge_status_decoded_electric_heater is None and "control_bridge_status_decoded_electric_heater" in self.model_fields_set:
            _dict['controlBridgeStatusDecodedElectricHeater'] = None

        # set to None if control_bridge_status_decoded_water_pump2 (nullable) is None
        # and model_fields_set contains the field
        if self.control_bridge_status_decoded_water_pump2 is None and "control_bridge_status_decoded_water_pump2" in self.model_fields_set:
            _dict['controlBridgeStatusDecodedWaterPump2'] = None

        # set to None if input_status (nullable) is None
        # and model_fields_set contains the field
        if self.input_status is None and "input_status" in self.model_fields_set:
            _dict['inputStatus'] = None

        # set to None if current_control_method (nullable) is None
        # and model_fields_set contains the field
        if self.current_control_method is None and "current_control_method" in self.model_fields_set:
            _dict['currentControlMethod'] = None

        # set to None if signal_strength (nullable) is None
        # and model_fields_set contains the field
        if self.signal_strength is None and "signal_strength" in self.model_fields_set:
            _dict['signalStrength'] = None

        # set to None if t1 (nullable) is None
        # and model_fields_set contains the field
        if self.t1 is None and "t1" in self.model_fields_set:
            _dict['t1'] = None

        # set to None if t2 (nullable) is None
        # and model_fields_set contains the field
        if self.t2 is None and "t2" in self.model_fields_set:
            _dict['t2'] = None

        # set to None if t_spare_ntc (nullable) is None
        # and model_fields_set contains the field
        if self.t_spare_ntc is None and "t_spare_ntc" in self.model_fields_set:
            _dict['tSpareNtc'] = None

        # set to None if t_board (nullable) is None
        # and model_fields_set contains the field
        if self.t_board is None and "t_board" in self.model_fields_set:
            _dict['tBoard'] = None

        # set to None if t_air_out (nullable) is None
        # and model_fields_set contains the field
        if self.t_air_out is None and "t_air_out" in self.model_fields_set:
            _dict['tAirOut'] = None

        # set to None if rpm (nullable) is None
        # and model_fields_set contains the field
        if self.rpm is None and "rpm" in self.model_fields_set:
            _dict['rpm'] = None

        # set to None if rpm_limiter (nullable) is None
        # and model_fields_set contains the field
        if self.rpm_limiter is None and "rpm_limiter" in self.model_fields_set:
            _dict['rpmLimiter'] = None

        # set to None if rpm_limiter_type (nullable) is None
        # and model_fields_set contains the field
        if self.rpm_limiter_type is None and "rpm_limiter_type" in self.model_fields_set:
            _dict['rpmLimiterType'] = None

        # set to None if p_compressor_in (nullable) is None
        # and model_fields_set contains the field
        if self.p_compressor_in is None and "p_compressor_in" in self.model_fields_set:
            _dict['pCompressorIn'] = None

        # set to None if p_compressor_out (nullable) is None
        # and model_fields_set contains the field
        if self.p_compressor_out is None and "p_compressor_out" in self.model_fields_set:
            _dict['pCompressorOut'] = None

        # set to None if p_compressor_in_target (nullable) is None
        # and model_fields_set contains the field
        if self.p_compressor_in_target is None and "p_compressor_in_target" in self.model_fields_set:
            _dict['pCompressorInTarget'] = None

        # set to None if t_inverter (nullable) is None
        # and model_fields_set contains the field
        if self.t_inverter is None and "t_inverter" in self.model_fields_set:
            _dict['tInverter'] = None

        # set to None if t_compressor_in (nullable) is None
        # and model_fields_set contains the field
        if self.t_compressor_in is None and "t_compressor_in" in self.model_fields_set:
            _dict['tCompressorIn'] = None

        # set to None if t_compressor_out (nullable) is None
        # and model_fields_set contains the field
        if self.t_compressor_out is None and "t_compressor_out" in self.model_fields_set:
            _dict['tCompressorOut'] = None

        # set to None if t_compressor_in_transient (nullable) is None
        # and model_fields_set contains the field
        if self.t_compressor_in_transient is None and "t_compressor_in_transient" in self.model_fields_set:
            _dict['tCompressorInTransient'] = None

        # set to None if t_compressor_out_transient (nullable) is None
        # and model_fields_set contains the field
        if self.t_compressor_out_transient is None and "t_compressor_out_transient" in self.model_fields_set:
            _dict['tCompressorOutTransient'] = None

        # set to None if delta_t_compressor_in_superheat (nullable) is None
        # and model_fields_set contains the field
        if self.delta_t_compressor_in_superheat is None and "delta_t_compressor_in_superheat" in self.model_fields_set:
            _dict['deltaTCompressorInSuperheat'] = None

        # set to None if compressor_power_low_accuracy (nullable) is None
        # and model_fields_set contains the field
        if self.compressor_power_low_accuracy is None and "compressor_power_low_accuracy" in self.model_fields_set:
            _dict['compressorPowerLowAccuracy'] = None

        # set to None if fan (nullable) is None
        # and model_fields_set contains the field
        if self.fan is None and "fan" in self.model_fields_set:
            _dict['fan'] = None

        # set to None if fan_power (nullable) is None
        # and model_fields_set contains the field
        if self.fan_power is None and "fan_power" in self.model_fields_set:
            _dict['fanPower'] = None

        # set to None if valve (nullable) is None
        # and model_fields_set contains the field
        if self.valve is None and "valve" in self.model_fields_set:
            _dict['valve'] = None

        # set to None if cm_mass_power_in (nullable) is None
        # and model_fields_set contains the field
        if self.cm_mass_power_in is None and "cm_mass_power_in" in self.model_fields_set:
            _dict['cmMassPowerIn'] = None

        # set to None if cm_mass_power_out (nullable) is None
        # and model_fields_set contains the field
        if self.cm_mass_power_out is None and "cm_mass_power_out" in self.model_fields_set:
            _dict['cmMassPowerOut'] = None

        # set to None if temperature_error_integral (nullable) is None
        # and model_fields_set contains the field
        if self.temperature_error_integral is None and "temperature_error_integral" in self.model_fields_set:
            _dict['temperatureErrorIntegral'] = None

        # set to None if thermostat_status (nullable) is None
        # and model_fields_set contains the field
        if self.thermostat_status is None and "thermostat_status" in self.model_fields_set:
            _dict['thermostatStatus'] = None

        # set to None if t_room (nullable) is None
        # and model_fields_set contains the field
        if self.t_room is None and "t_room" in self.model_fields_set:
            _dict['tRoom'] = None

        # set to None if t_room_target (nullable) is None
        # and model_fields_set contains the field
        if self.t_room_target is None and "t_room_target" in self.model_fields_set:
            _dict['tRoomTarget'] = None

        # set to None if ot_boiler_status (nullable) is None
        # and model_fields_set contains the field
        if self.ot_boiler_status is None and "ot_boiler_status" in self.model_fields_set:
            _dict['otBoilerStatus'] = None

        # set to None if ot_boiler_feed_temperature (nullable) is None
        # and model_fields_set contains the field
        if self.ot_boiler_feed_temperature is None and "ot_boiler_feed_temperature" in self.model_fields_set:
            _dict['otBoilerFeedTemperature'] = None

        # set to None if ot_boiler_return_temperature (nullable) is None
        # and model_fields_set contains the field
        if self.ot_boiler_return_temperature is None and "ot_boiler_return_temperature" in self.model_fields_set:
            _dict['otBoilerReturnTemperature'] = None

        # set to None if inverter_input_voltage (nullable) is None
        # and model_fields_set contains the field
        if self.inverter_input_voltage is None and "inverter_input_voltage" in self.model_fields_set:
            _dict['inverterInputVoltage'] = None

        # set to None if central_heating_pwm_requested_duty_cycle (nullable) is None
        # and model_fields_set contains the field
        if self.central_heating_pwm_requested_duty_cycle is None and "central_heating_pwm_requested_duty_cycle" in self.model_fields_set:
            _dict['centralHeatingPwmRequestedDutyCycle'] = None

        # set to None if central_heating_flow (nullable) is None
        # and model_fields_set contains the field
        if self.central_heating_flow is None and "central_heating_flow" in self.model_fields_set:
            _dict['centralHeatingFlow'] = None

        # set to None if dhw_pwm_requested_duty_cycle (nullable) is None
        # and model_fields_set contains the field
        if self.dhw_pwm_requested_duty_cycle is None and "dhw_pwm_requested_duty_cycle" in self.model_fields_set:
            _dict['dhwPwmRequestedDutyCycle'] = None

        # set to None if dhw_flow (nullable) is None
        # and model_fields_set contains the field
        if self.dhw_flow is None and "dhw_flow" in self.model_fields_set:
            _dict['dhwFlow'] = None

        # set to None if indoor_unit_heater_temperature (nullable) is None
        # and model_fields_set contains the field
        if self.indoor_unit_heater_temperature is None and "indoor_unit_heater_temperature" in self.model_fields_set:
            _dict['indoorUnitHeaterTemperature'] = None

        # set to None if indoor_unit_input_current (nullable) is None
        # and model_fields_set contains the field
        if self.indoor_unit_input_current is None and "indoor_unit_input_current" in self.model_fields_set:
            _dict['indoorUnitInputCurrent'] = None

        # set to None if sinr (nullable) is None
        # and model_fields_set contains the field
        if self.sinr is None and "sinr" in self.model_fields_set:
            _dict['sinr'] = None

        # set to None if cooling_status (nullable) is None
        # and model_fields_set contains the field
        if self.cooling_status is None and "cooling_status" in self.model_fields_set:
            _dict['coolingStatus'] = None

        # set to None if debug_variable1 (nullable) is None
        # and model_fields_set contains the field
        if self.debug_variable1 is None and "debug_variable1" in self.model_fields_set:
            _dict['debugVariable1'] = None

        # set to None if debug_variable2 (nullable) is None
        # and model_fields_set contains the field
        if self.debug_variable2 is None and "debug_variable2" in self.model_fields_set:
            _dict['debugVariable2'] = None

        # set to None if debug_variable3 (nullable) is None
        # and model_fields_set contains the field
        if self.debug_variable3 is None and "debug_variable3" in self.model_fields_set:
            _dict['debugVariable3'] = None

        # set to None if debug_variable4 (nullable) is None
        # and model_fields_set contains the field
        if self.debug_variable4 is None and "debug_variable4" in self.model_fields_set:
            _dict['debugVariable4'] = None

        # set to None if debug_variable5 (nullable) is None
        # and model_fields_set contains the field
        if self.debug_variable5 is None and "debug_variable5" in self.model_fields_set:
            _dict['debugVariable5'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of RawHeatPumpLogDto from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "heatPumpId": obj.get("heatPumpId"),
            "timestamp": obj.get("timestamp"),
            "state": obj.get("state"),
            "rpmCtrl": obj.get("rpmCtrl"),
            "error": obj.get("error"),
            "errorDecodedDtcNone": obj.get("errorDecodedDtcNone"),
            "errorDecodedDtcContinue": obj.get("errorDecodedDtcContinue"),
            "errorDecodedDtcCompressorOff": obj.get("errorDecodedDtcCompressorOff"),
            "errorDecodedDtcDefrostForbidden": obj.get("errorDecodedDtcDefrostForbidden"),
            "errorDecodedDtcRequestService": obj.get("errorDecodedDtcRequestService"),
            "errorDecodedDtcUseHeatingCurve": obj.get("errorDecodedDtcUseHeatingCurve"),
            "errorDecodedDtcDhwForbidden": obj.get("errorDecodedDtcDhwForbidden"),
            "errorDecodedDtcError": obj.get("errorDecodedDtcError"),
            "errorDecodedDtcInactive": obj.get("errorDecodedDtcInactive"),
            "controlBridgeStatus": obj.get("controlBridgeStatus"),
            "controlBridgeStatusDecodedWaterPump": obj.get("controlBridgeStatusDecodedWaterPump"),
            "controlBridgeStatusDecodedDhwValve": obj.get("controlBridgeStatusDecodedDhwValve"),
            "controlBridgeStatusDecodedGasBoiler": obj.get("controlBridgeStatusDecodedGasBoiler"),
            "controlBridgeStatusDecodedElectricHeater": obj.get("controlBridgeStatusDecodedElectricHeater"),
            "controlBridgeStatusDecodedWaterPump2": obj.get("controlBridgeStatusDecodedWaterPump2"),
            "inputStatus": obj.get("inputStatus"),
            "currentControlMethod": obj.get("currentControlMethod"),
            "signalStrength": obj.get("signalStrength"),
            "t1": obj.get("t1"),
            "t2": obj.get("t2"),
            "tSpareNtc": obj.get("tSpareNtc"),
            "tBoard": obj.get("tBoard"),
            "tAirIn": obj.get("tAirIn"),
            "tAirOut": obj.get("tAirOut"),
            "tWaterIn": obj.get("tWaterIn"),
            "tWaterOut": obj.get("tWaterOut"),
            "tWaterHouseIn": obj.get("tWaterHouseIn"),
            "rpm": obj.get("rpm"),
            "rpmLimiter": obj.get("rpmLimiter"),
            "rpmLimiterType": obj.get("rpmLimiterType"),
            "pCompressorIn": obj.get("pCompressorIn"),
            "pCompressorOut": obj.get("pCompressorOut"),
            "pCompressorInTarget": obj.get("pCompressorInTarget"),
            "tInverter": obj.get("tInverter"),
            "tCompressorIn": obj.get("tCompressorIn"),
            "tCompressorOut": obj.get("tCompressorOut"),
            "tCompressorInTransient": obj.get("tCompressorInTransient"),
            "tCompressorOutTransient": obj.get("tCompressorOutTransient"),
            "deltaTCompressorInSuperheat": obj.get("deltaTCompressorInSuperheat"),
            "compressorPowerLowAccuracy": obj.get("compressorPowerLowAccuracy"),
            "fan": obj.get("fan"),
            "fanPower": obj.get("fanPower"),
            "valve": obj.get("valve"),
            "cmMassPowerIn": obj.get("cmMassPowerIn"),
            "cmMassPowerOut": obj.get("cmMassPowerOut"),
            "temperatureErrorIntegral": obj.get("temperatureErrorIntegral"),
            "onOffThermostatState": obj.get("onOffThermostatState"),
            "thermostatStatus": obj.get("thermostatStatus"),
            "tRoom": obj.get("tRoom"),
            "tRoomTarget": obj.get("tRoomTarget"),
            "tThermostatSetpoint": obj.get("tThermostatSetpoint"),
            "otBoilerStatus": obj.get("otBoilerStatus"),
            "otBoilerFeedTemperature": obj.get("otBoilerFeedTemperature"),
            "otBoilerReturnTemperature": obj.get("otBoilerReturnTemperature"),
            "inverterInputVoltage": obj.get("inverterInputVoltage"),
            "centralHeatingPwmRequestedDutyCycle": obj.get("centralHeatingPwmRequestedDutyCycle"),
            "centralHeatingFlow": obj.get("centralHeatingFlow"),
            "dhwPwmRequestedDutyCycle": obj.get("dhwPwmRequestedDutyCycle"),
            "dhwFlow": obj.get("dhwFlow"),
            "indoorUnitHeaterTemperature": obj.get("indoorUnitHeaterTemperature"),
            "indoorUnitInputCurrent": obj.get("indoorUnitInputCurrent"),
            "sinr": obj.get("sinr"),
            "coolingStatus": obj.get("coolingStatus"),
            "debugVariable1": obj.get("debugVariable1"),
            "debugVariable2": obj.get("debugVariable2"),
            "debugVariable3": obj.get("debugVariable3"),
            "debugVariable4": obj.get("debugVariable4"),
            "debugVariable5": obj.get("debugVariable5"),
            "interval": obj.get("interval")
        })
        return _obj





# coding: utf-8

"""
    Weheat Backend

    This is the backend for the Weheat project

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from typing import Any, ClassVar, Dict, List, Optional
from pydantic import BaseModel, StrictStr
from pydantic import Field
from weheat.models.role import Role
try:
    from typing import Self
except ImportError:
    from typing_extensions import Self

class ReadUserDto(BaseModel):
    """
    ReadUserDto
    """ # noqa: E501
    id: StrictStr = Field(description="Identifier of the user")
    first_name: Optional[StrictStr] = Field(default=None, description="First name of the user if available", alias="firstName")
    last_name: Optional[StrictStr] = Field(default=None, description="Last Name of the user if available", alias="lastName")
    role: Role
    email: Optional[StrictStr] = Field(default=None, description="Email address of the user")
    updated_on: datetime = Field(description="Timestamp of the last update to the user entry", alias="updatedOn")
    created_on: datetime = Field(description="Timestamp of the creation of the user entry", alias="createdOn")
    __properties: ClassVar[List[str]] = ["id", "firstName", "lastName", "role", "email", "updatedOn", "createdOn"]

    model_config = {
        "populate_by_name": True,
        "validate_assignment": True,
        "protected_namespaces": (),
    }


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ReadUserDto from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={
            },
            exclude_none=True,
        )
        # set to None if first_name (nullable) is None
        # and model_fields_set contains the field
        if self.first_name is None and "first_name" in self.model_fields_set:
            _dict['firstName'] = None

        # set to None if last_name (nullable) is None
        # and model_fields_set contains the field
        if self.last_name is None and "last_name" in self.model_fields_set:
            _dict['lastName'] = None

        # set to None if email (nullable) is None
        # and model_fields_set contains the field
        if self.email is None and "email" in self.model_fields_set:
            _dict['email'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of ReadUserDto from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "firstName": obj.get("firstName"),
            "lastName": obj.get("lastName"),
            "role": obj.get("role"),
            "email": obj.get("email"),
            "updatedOn": obj.get("updatedOn"),
            "createdOn": obj.get("createdOn")
        })
        return _obj





# coding: utf-8

"""
    Weheat Backend

    This is the backend for the Weheat project

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
import pprint
import re  # noqa: F401
from enum import Enum



try:
    from typing import Self
except ImportError:
    from typing_extensions import Self


class DeviceState(int, Enum):
    """
    State of the device:   - ProductionObsoleteState (0),   - InStock (1),   - Sold (2),   - Active (3),   - Inactive (4),   - Broken (5),   - Test (6)   - Configured (7)  For now Production, InStock and Sold will only be applicable to Devices that can be actually installed in installation
    """

    """
    allowed enum values
    """
    NUMBER_0 = 0
    NUMBER_1 = 1
    NUMBER_2 = 2
    NUMBER_3 = 3
    NUMBER_4 = 4
    NUMBER_5 = 5
    NUMBER_6 = 6
    NUMBER_7 = 7

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of DeviceState from a JSON string"""
        return cls(json.loads(json_str))





import asyncio
from typing import Union

import aiohttp

from weheat.configuration import Configuration
from weheat.api_client import ApiClient
from weheat.api.user_api import UserApi


async def async_get_user_id_from_token(api_url: str, access_token: str, client_session:aiohttp.ClientSession|None = None) -> Union[str,None]:
    """ Get the user id from the current logged-in user. """
    config = Configuration(host=api_url, access_token=access_token, client_session=client_session)

    async with ApiClient(configuration=config) as client:
        response = await UserApi(
            client
        ).api_v1_users_me_get_with_http_info()

        if response.status_code == 200:
            return response.data.id # type: ignore[no-any-return]
        return None




# coding: utf-8

"""
    Weheat Backend

    This is the backend for the Weheat project

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
import pprint
import re  # noqa: F401
from enum import Enum



try:
    from typing import Self
except ImportError:
    from typing_extensions import Self


class BoilerType(int, Enum):
    """
    Type of model and use of the heat pump   - Unknown: Not known if a boiler is in the installation (0)   - None: No boiler detected (1)   - OnOff: On/off boiler detected (2)   - OpenTherm: OpenTherm boiler detected (3)
    """

    """
    allowed enum values
    """
    NUMBER_0 = 0
    NUMBER_1 = 1
    NUMBER_2 = 2
    NUMBER_3 = 3

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of BoilerType from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    Weheat Backend

    This is the backend for the Weheat project

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import io
import warnings

from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Dict, List, Optional, Tuple, Union, Any

try:
    from typing import Annotated
except ImportError:
    from typing_extensions import Annotated

from pydantic import Field
from typing_extensions import Annotated
from pydantic import StrictInt, StrictStr

from typing import List, Optional

from weheat.models.device_state import DeviceState
from weheat.models.read_all_heat_pump_dto import ReadAllHeatPumpDto
from weheat.models.read_heat_pump_dto import ReadHeatPumpDto

from weheat.api_client import ApiClient
from weheat.api_response import ApiResponse
from weheat.rest import RESTResponseType


class HeatPumpApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    async def api_v1_heat_pumps_get(
        self,
        search: Annotated[Optional[StrictStr], Field(description="String with keywords (split by spaces) to search on SN, PN and Name of a heat pump")] = None,
        page: Annotated[Optional[StrictInt], Field(description="The page number")] = None,
        page_size: Annotated[Optional[StrictInt], Field(description="The page size")] = None,
        state: Annotated[Optional[DeviceState], Field(description="Filter for which device state the heat pump should be in using Device")] = None,
        generic_model: Annotated[Optional[StrictStr], Field(description="Filter for which generic model the heat pump should have")] = None,
        x_version: Annotated[Optional[StrictStr], Field(description="Optional version parameter for frontend applications to check if an update / refresh is needed")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[ReadAllHeatPumpDto]:
        """Gets all heat pumps (Paged, Searchable)


        :param search: String with keywords (split by spaces) to search on SN, PN and Name of a heat pump
        :type search: str
        :param page: The page number
        :type page: int
        :param page_size: The page size
        :type page_size: int
        :param state: Filter for which device state the heat pump should be in using Device
        :type state: DeviceState
        :param generic_model: Filter for which generic model the heat pump should have
        :type generic_model: str
        :param x_version: Optional version parameter for frontend applications to check if an update / refresh is needed
        :type x_version: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._api_v1_heat_pumps_get_serialize(
            search=search,
            page=page,
            page_size=page_size,
            state=state,
            generic_model=generic_model,
            x_version=x_version,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '403': "str",
            '500': None,
            '505': None,
            '200': "List[ReadAllHeatPumpDto]",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    async def api_v1_heat_pumps_get_with_http_info(
        self,
        search: Annotated[Optional[StrictStr], Field(description="String with keywords (split by spaces) to search on SN, PN and Name of a heat pump")] = None,
        page: Annotated[Optional[StrictInt], Field(description="The page number")] = None,
        page_size: Annotated[Optional[StrictInt], Field(description="The page size")] = None,
        state: Annotated[Optional[DeviceState], Field(description="Filter for which device state the heat pump should be in using Device")] = None,
        generic_model: Annotated[Optional[StrictStr], Field(description="Filter for which generic model the heat pump should have")] = None,
        x_version: Annotated[Optional[StrictStr], Field(description="Optional version parameter for frontend applications to check if an update / refresh is needed")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[ReadAllHeatPumpDto]]:
        """Gets all heat pumps (Paged, Searchable)


        :param search: String with keywords (split by spaces) to search on SN, PN and Name of a heat pump
        :type search: str
        :param page: The page number
        :type page: int
        :param page_size: The page size
        :type page_size: int
        :param state: Filter for which device state the heat pump should be in using Device
        :type state: DeviceState
        :param generic_model: Filter for which generic model the heat pump should have
        :type generic_model: str
        :param x_version: Optional version parameter for frontend applications to check if an update / refresh is needed
        :type x_version: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._api_v1_heat_pumps_get_serialize(
            search=search,
            page=page,
            page_size=page_size,
            state=state,
            generic_model=generic_model,
            x_version=x_version,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '403': "str",
            '500': None,
            '505': None,
            '200': "List[ReadAllHeatPumpDto]",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    async def api_v1_heat_pumps_get_without_preload_content(
        self,
        search: Annotated[Optional[StrictStr], Field(description="String with keywords (split by spaces) to search on SN, PN and Name of a heat pump")] = None,
        page: Annotated[Optional[StrictInt], Field(description="The page number")] = None,
        page_size: Annotated[Optional[StrictInt], Field(description="The page size")] = None,
        state: Annotated[Optional[DeviceState], Field(description="Filter for which device state the heat pump should be in using Device")] = None,
        generic_model: Annotated[Optional[StrictStr], Field(description="Filter for which generic model the heat pump should have")] = None,
        x_version: Annotated[Optional[StrictStr], Field(description="Optional version parameter for frontend applications to check if an update / refresh is needed")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Gets all heat pumps (Paged, Searchable)


        :param search: String with keywords (split by spaces) to search on SN, PN and Name of a heat pump
        :type search: str
        :param page: The page number
        :type page: int
        :param page_size: The page size
        :type page_size: int
        :param state: Filter for which device state the heat pump should be in using Device
        :type state: DeviceState
        :param generic_model: Filter for which generic model the heat pump should have
        :type generic_model: str
        :param x_version: Optional version parameter for frontend applications to check if an update / refresh is needed
        :type x_version: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._api_v1_heat_pumps_get_serialize(
            search=search,
            page=page,
            page_size=page_size,
            state=state,
            generic_model=generic_model,
            x_version=x_version,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '403': "str",
            '500': None,
            '505': None,
            '200': "List[ReadAllHeatPumpDto]",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _api_v1_heat_pumps_get_serialize(
        self,
        search,
        page,
        page_size,
        state,
        generic_model,
        x_version,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> Tuple:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if search is not None:
            
            _query_params.append(('search', search))
            
        if page is not None:
            
            _query_params.append(('page', page))
            
        if page_size is not None:
            
            _query_params.append(('pageSize', page_size))
            
        if state is not None:
            
            _query_params.append(('state', state.value))
            
        if generic_model is not None:
            
            _query_params.append(('genericModel', generic_model))
            
        # process the header parameters
        if x_version is not None:
            _header_params['x-version'] = x_version
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            [
                'text/plain', 
                'application/json', 
                'text/json'
            ]
        )


        # authentication setting
        _auth_settings: List[str] = [
            'oauth2'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/v1/heat-pumps',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )


    @validate_call
    async def api_v1_heat_pumps_heat_pump_id_get(
        self,
        heat_pump_id: Annotated[StrictStr, Field(description="The heatPumpId of the heat pump you want to get")],
        x_version: Annotated[Optional[StrictStr], Field(description="Optional version parameter for frontend applications to check if an update / refresh is needed")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ReadHeatPumpDto:
        """Gets a heat pump by heatPumpId


        :param heat_pump_id: The heatPumpId of the heat pump you want to get (required)
        :type heat_pump_id: str
        :param x_version: Optional version parameter for frontend applications to check if an update / refresh is needed
        :type x_version: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._api_v1_heat_pumps_heat_pump_id_get_serialize(
            heat_pump_id=heat_pump_id,
            x_version=x_version,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '403': "str",
            '500': None,
            '505': None,
            '200': "ReadHeatPumpDto",
            '404': None,
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    async def api_v1_heat_pumps_heat_pump_id_get_with_http_info(
        self,
        heat_pump_id: Annotated[StrictStr, Field(description="The heatPumpId of the heat pump you want to get")],
        x_version: Annotated[Optional[StrictStr], Field(description="Optional version parameter for frontend applications to check if an update / refresh is needed")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[ReadHeatPumpDto]:
        """Gets a heat pump by heatPumpId


        :param heat_pump_id: The heatPumpId of the heat pump you want to get (required)
        :type heat_pump_id: str
        :param x_version: Optional version parameter for frontend applications to check if an update / refresh is needed
        :type x_version: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._api_v1_heat_pumps_heat_pump_id_get_serialize(
            heat_pump_id=heat_pump_id,
            x_version=x_version,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '403': "str",
            '500': None,
            '505': None,
            '200': "ReadHeatPumpDto",
            '404': None,
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    async def api_v1_heat_pumps_heat_pump_id_get_without_preload_content(
        self,
        heat_pump_id: Annotated[StrictStr, Field(description="The heatPumpId of the heat pump you want to get")],
        x_version: Annotated[Optional[StrictStr], Field(description="Optional version parameter for frontend applications to check if an update / refresh is needed")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Gets a heat pump by heatPumpId


        :param heat_pump_id: The heatPumpId of the heat pump you want to get (required)
        :type heat_pump_id: str
        :param x_version: Optional version parameter for frontend applications to check if an update / refresh is needed
        :type x_version: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._api_v1_heat_pumps_heat_pump_id_get_serialize(
            heat_pump_id=heat_pump_id,
            x_version=x_version,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '403': "str",
            '500': None,
            '505': None,
            '200': "ReadHeatPumpDto",
            '404': None,
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _api_v1_heat_pumps_heat_pump_id_get_serialize(
        self,
        heat_pump_id,
        x_version,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> Tuple:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if heat_pump_id is not None:
            _path_params['heatPumpId'] = heat_pump_id
        # process the query parameters
        # process the header parameters
        if x_version is not None:
            _header_params['x-version'] = x_version
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            [
                'text/plain', 
                'application/json', 
                'text/json'
            ]
        )


        # authentication setting
        _auth_settings: List[str] = [
            'oauth2'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/v1/heat-pumps/{heatPumpId}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





import pytest
from pydantic import ValidationError

from weheat.abstractions.heat_pump import HeatPump

from weheat.abstractions.discovery import HeatPumpDiscovery
from weheat.abstractions.user import async_get_user_id_from_token
from weheat.exceptions import UnauthorizedException, ForbiddenException


uuid_list = [
    '54ad613f-48e5-4583-b0e0-1a96d29131fa',
    '306625f3-124a-42eb-8e31-952963c65932',
    '573852f6-029f-4106-a019-2183b14a351f'
]

@pytest.mark.asyncio
async def test_discovery(api_fixture):
    discovery = await HeatPumpDiscovery.async_discover_active(api_url=api_fixture.api_url, access_token=api_fixture.access_token)

    #The account is configured to find at least one Blackbird, one sparrow and one flint
    assert len(discovery) >= 3

    #check some information for each pump, find it by type in the list
    blackbird = [hp for hp in discovery if 'blackbird' in hp.model.lower()]
    sparrow = [hp for hp in discovery if 'sparrow' in hp.model.lower()]
    flint = [hp for hp in discovery if 'flint' in hp.model.lower()]

    # sanity check lengths
    assert len(blackbird) >= 1
    assert len(sparrow) >= 1
    assert len(flint) >= 1

    blackbird = blackbird[0]
    sparrow = sparrow[0]
    flint = flint[0]

    # must have a UUID
    assert len(blackbird.uuid) == 36
    assert len(sparrow.uuid) == 36
    assert len(flint.uuid) == 36

    # must have a serial number
    assert len(blackbird.sn) == 12
    assert len(sparrow.sn) == 12
    assert len(flint.sn) == 12

@pytest.mark.asyncio
async def test_user(api_fixture):
    user = await async_get_user_id_from_token(api_fixture.api_url, api_fixture.access_token)

    #check we got a UUID
    assert len(user) == 36

@pytest.mark.parametrize("uuid", uuid_list)
@pytest.mark.asyncio
async def test_hp_log(api_fixture, uuid):
    heatpump = HeatPump(api_fixture.api_url, uuid)

    # if the API is changed, this call will fail on a python validation error
    try:
        await heatpump.async_get_status(api_fixture.access_token)
    except ValidationError as e:
        print(f'Validation unsuccessful: {e}')
        pytest.fail('Validation failed')

    except UnauthorizedException as e:
        pytest.fail('get_status call failed')
    except ForbiddenException as e:
        pytest.fail('get_status call failed')
    except Exception as e:
        pytest.fail('get_status call failed')

    # check that all core values are present
    assert heatpump.water_inlet_temperature >= 10
    assert heatpump.water_outlet_temperature >= 10
    assert heatpump.air_inlet_temperature >= -20
    if heatpump.air_outlet_temperature is not None: # flint does not have an air out sensor
        assert heatpump.air_outlet_temperature >= -20
    if heatpump.water_house_in_temperature is not None: # indoor unit may be disconnected
        assert heatpump.water_house_in_temperature >= -1
    if heatpump.thermostat_room_temperature_setpoint is not None: #thermostat may be disconnected
        assert heatpump.thermostat_room_temperature_setpoint >= -1
    if heatpump.thermostat_room_temperature is not None:  #thermostat may be disconnected
        assert heatpump.thermostat_room_temperature >= -1
    assert heatpump.power_input >= 0
    assert heatpump.power_output >= 0
    assert heatpump.compressor_rpm >= 0
    assert heatpump.heat_pump_state in HeatPump.State
    assert heatpump.energy_total >= 0
    assert heatpump.energy_output >= 0









import asyncio
from dataclasses import dataclass

import aiohttp

from weheat import DeviceState
from weheat.configuration import Configuration
from weheat.api_client import ApiClient
from weheat.api.heat_pump_api import HeatPumpApi


class HeatPumpDiscovery:
    @dataclass
    class HeatPumpInfo:
        uuid: str
        name: str
        model: str
        sn : str
        has_dhw: bool = False

    @staticmethod
    async def async_discover_active(api_url: str, access_token: str, client_session:aiohttp.ClientSession|None = None) -> list[HeatPumpInfo]:
        discovered_pumps = []

        config = Configuration(host=api_url, access_token=access_token, client_session=client_session)

        async with ApiClient(configuration=config) as client:

            response = await  HeatPumpApi(client).api_v1_heat_pumps_get_with_http_info('', 1, 1000, DeviceState.NUMBER_3)

            if response.status_code == 200:
                for pump in response.data:
                    # Model of the heat pump
                    # - BlackBirdP80: BlackBird P80 heat pump (0)
                    # - BlackBirdP60: BlackBird P60 heat pump (1)
                    # - SparrowP60Brown: Sparrow P60 heat pump, colour brown (default) (2)
                    # - SparrowP60Green: Sparrow P60 heat pump, colour green (3)
                    # - SparrowP60Grey: Sparrow P60 heat pump, colour grey (4)
                    # - FlintP40: Flint P40 heat pump (5)
                    model_string = "Blackbird P80 heat pump"
                    if pump.model == 1:
                        model_string = "Blackbird P60 heat pump"
                    elif 2 <= pump.model <= 4:
                        model_string = "Sparrow P60 heat pump"
                    elif pump.model == 5:
                        model_string = "Flint P40 heat pump"


                    dhw = False
                    if pump.dhw_type is not None and pump.dhw_type == 1:
                        dhw = True

                    discovered_pumps.append(
                        HeatPumpDiscovery.HeatPumpInfo(
                            uuid=pump.id,
                            name=pump.name,
                            model=model_string,
                            sn=pump.serial_number,
                            has_dhw=dhw,
                        )
                    )
        return discovered_pumps


"""Weheat heat pump abstraction from the API."""
import asyncio
from enum import Enum, auto

import aiohttp

from weheat import HeatPumpApi
from weheat.configuration import Configuration
from weheat.api_client import ApiClient
from weheat.api.heat_pump_log_api import HeatPumpLogApi
from weheat.api.energy_log_api import EnergyLogApi
from datetime import datetime, timedelta
from typing import TypeVar, Union, Optional

# before this date no energy logs are available, so start from this point onwards
START_DATE = datetime(2023, 1, 1, 0, 0, 0)

T = TypeVar("T", bool, int, float)


class HeatPump:
    """Heat pump class representing a heat pump."""

    class State(Enum):
        STANDBY = auto()
        WATER_CHECK = auto()
        HEATING = auto()
        COOLING = auto()
        DHW = auto()
        LEGIONELLA_PREVENTION = auto()
        DEFROSTING = auto()
        SELF_TEST = auto()
        MANUAL_CONTROL = auto()

    def __init__(self, api_url: str, uuid: str, client_session: aiohttp.ClientSession | None = None) -> None:
        self._api_url = api_url
        self._uuid = uuid
        self._last_log = None
        self._energy_consumption: Union[float, None] = None
        self._energy_output: Union[float, None] = None
        self._nominal_max_power: Union[float, None] = None
        self._client = client_session

    async def async_get_status(self, access_token: str) -> None:
        """Updates the heat pump instance with data from the API."""
        await self.async_get_logs(access_token)
        await self.async_get_energy(access_token)
        
        
    async def async_get_logs(self, access_token: str) -> None:
        """Updates the heat pump instance with data from the API."""
        try:
            config = Configuration(host=self._api_url, access_token=access_token, client_session=self._client)

            async with ApiClient(configuration=config) as client:
                # Set the max power once
                if self._nominal_max_power is None:
                    response = await HeatPumpApi(client).api_v1_heat_pumps_heat_pump_id_get_with_http_info(
                        heat_pump_id=self._uuid)

                    if response.status_code == 200:
                        self._set_nominal_max_power_for_model(response.data.model)

                response = await HeatPumpLogApi(
                    client
                ).api_v1_heat_pumps_heat_pump_id_logs_latest_get_with_http_info(
                    heat_pump_id=self._uuid)

                if response.status_code == 200:
                    self._last_log = response.data


        except Exception as e:
            self._last_log = None
            raise e


    async def async_get_energy(self, access_token: str) -> None:
        """Updates the heat pump instance with data from the API."""
        try:
            config = Configuration(host=self._api_url, access_token=access_token, client_session=self._client)

            async with ApiClient(configuration=config) as client:
                # Also get all energy totals form past years and add them together
                # As end time pick today + 1 day to avoid issues with timezones
                response = await EnergyLogApi(client).api_v1_energy_logs_heat_pump_id_get_with_http_info(
                    heat_pump_id=self._uuid,
                    start_time=START_DATE,
                    end_time=datetime.now() + timedelta(days=1),
                    interval='Month')

                if response.status_code == 200:
                    # aggregate the energy consumption
                    self._energy_consumption = 0
                    self._energy_output = 0
                    for year in response.data:
                        self._energy_consumption += year.total_ein_cooling
                        self._energy_consumption += year.total_ein_heating
                        self._energy_consumption += year.total_ein_heating_defrost
                        self._energy_consumption += year.total_ein_dhw
                        self._energy_consumption += year.total_ein_dhw_defrost
                        self._energy_output += year.total_e_out_cooling
                        self._energy_output += year.total_e_out_heating
                        self._energy_output += year.total_e_out_heating_defrost
                        self._energy_output += year.total_e_out_dhw
                        self._energy_output += year.total_e_out_dhw_defrost

        except Exception as e:
            self._energy_consumption = None
            self._energy_output = None

    def _if_available(self, key: str) -> Optional[T]:
        """Returns the value from the last logged value if available. None otherwise."""
        if self._last_log is not None and hasattr(self._last_log, key):
            return getattr(self._last_log, key)
        return None

    def _set_nominal_max_power_for_model(self, model_id: int) -> None:
        # These numbers are the rpm at 100% power in the portal
        # RPM can go above 100% if the limit is increased in the portal.
        # except for the Flint, that cannot go above 100%.
        if model_id == 1:
            # BB60
            self._nominal_max_power = 5280
        elif 2 <= model_id <= 4:
            # SP60
            self._nominal_max_power = 5280
        elif model_id == 5:
            # Flint
            self._nominal_max_power = 5400
        else:
            # BB80
            self._nominal_max_power = 4500

    def __str__(self) -> str:
        return f"WeheatHeatPump(uuid={self._uuid}, last update={self._if_available('timestamp')})"

    def __repr__(self) -> str:
        return self.__str__()

    @property
    def raw_content(self) -> Optional[dict]:
        if self._last_log is not None:
            return vars(self._last_log) # type: ignore[unreachable]
        return None

    @property
    def water_inlet_temperature(self) -> Union[float, None]:
        """The heat pump water inlet temperature."""
        return self._if_available("t_water_in")

    @property
    def water_outlet_temperature(self) -> Union[float, None]:
        """The heat pump water outlet temperature."""
        return self._if_available("t_water_out")

    @property
    def water_house_in_temperature(self) -> Union[float, None]:
        """The water house in temperature."""
        return self._if_available("t_water_house_in")

    @property
    def air_inlet_temperature(self) -> Union[float, None]:
        """The heat pump air inlet temperature."""
        return self._if_available("t_air_in")

    @property
    def air_outlet_temperature(self) -> Union[float, None]:
        """The heat pump air outlet temperature."""
        return self._if_available("t_air_out")

    @property
    def thermostat_water_setpoint(self) -> Union[float, None]:
        """The  thermostat water setpoint."""
        return self._if_available("t_thermostat_setpoint")

    @property
    def thermostat_room_temperature(self) -> Union[float, None]:
        """The thermostat current room temperature."""
        return self._if_available("t_room")

    @property
    def thermostat_room_temperature_setpoint(self) -> Union[float, None]:
        """The thermostat room temperature setpoint."""
        return self._if_available("t_room_target")

    @property
    def thermostat_on_off_state(self) -> Union[bool, None]:
        """The thermostat on/off state."""
        return self._if_available("on_off_thermostat_state")

    @property
    def power_input(self) -> Union[float, None]:
        """The heat pumps power input."""
        return self._if_available("cm_mass_power_in")

    @property
    def power_output(self) -> Union[float, None]:
        """The heat pumps hydraulic output power."""
        return self._if_available("cm_mass_power_out")

    @property
    def dhw_top_temperature(self) -> Union[float, None]:
        """The DHW vessel top temperature."""
        return self._if_available("t1")

    @property
    def dhw_bottom_temperature(self) -> Union[float, None]:
        """The DHW vessel bottom temperature."""
        return self._if_available("t2")

    @property
    def cop(self) -> Union[float, None]:
        """
        Returns the coefficient of performance of the heat pump.
        Note that this is calculated from a singular log entry and might not be accurate when the
        heat pump is changing its output power or switching states
        """
        input = self.power_input
        output = self.power_output
        if input is not None and output is not None and input != 0:
            return output / input
        return None

    @property
    def indoor_unit_water_pump_state(self) -> Union[bool, None]:
        """Decoded water pump state."""
        return self._if_available("control_bridge_status_decoded_water_pump")

    @property
    def indoor_unit_auxiliary_pump_state(self) -> Union[bool, None]:
        """Decoded auxiliary pump state."""
        return self._if_available("control_bridge_status_decoded_water_pump2")

    @property
    def indoor_unit_dhw_valve_or_pump_state(self) -> Union[bool, None]:
        """Decoded DHW valve or pump state."""
        return self._if_available("control_bridge_status_decoded_dhw_valve")

    @property
    def indoor_unit_gas_boiler_state(self) -> Union[bool, None]:
        """Decoded gas boiler state."""
        return self._if_available("control_bridge_status_decoded_gas_boiler")

    @property
    def indoor_unit_electric_heater_state(self) -> Union[bool, None]:
        """Decoded electric heater state."""
        return self._if_available("control_bridge_status_decoded_electric_heater")

    @property
    def compressor_percentage(self) -> Union[int, None]:
        current_rpm = self._if_available("rpm")
        if self._nominal_max_power is not None and current_rpm is not None:
            # calculate percentage of rpm
            return int((100 / self._nominal_max_power) * current_rpm)
        return None

    @property
    def compressor_rpm(self) -> Union[float, None]:
        """Compressor RPM."""
        return self._if_available("rpm")

    @property
    def heat_pump_state(self) -> Union[State, None]:
        """The heat pump state."""
        numeric_state = self._if_available("state")
        if numeric_state is None:
            return None

        if numeric_state == 40:
            return self.State.STANDBY
        elif numeric_state == 70:
            return self.State.HEATING
        elif numeric_state == 130:
            return self.State.COOLING
        elif numeric_state == 150:
            return self.State.DHW
        elif numeric_state == 160:
            return self.State.LEGIONELLA_PREVENTION
        elif numeric_state == 170:
            return self.State.SELF_TEST
        elif numeric_state == 180:
            return self.State.MANUAL_CONTROL
        elif numeric_state >= 200 and numeric_state <= 240:
            return self.State.DEFROSTING
        return None

    @property
    def energy_total(self) -> Union[float, None]:
        """The total used energy in kWh from 2023 to now."""
        return self._energy_consumption

    @property
    def energy_output(self) -> Union[float, None]:
        """The total energy output in kWh from 2023 to now."""
        return self._energy_output



# coding: utf-8

# flake8: noqa
"""
    Weheat Backend

    This is the backend for the Weheat project

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from weheat.models.boiler_type import BoilerType
from weheat.models.device_state import DeviceState
from weheat.models.dhw_type import DhwType
from weheat.models.energy_view_dto import EnergyViewDto
from weheat.models.heat_pump_log_view_dto import HeatPumpLogViewDto
from weheat.models.heat_pump_model import HeatPumpModel
from weheat.models.heat_pump_status_enum import HeatPumpStatusEnum
from weheat.models.heat_pump_type import HeatPumpType
from weheat.models.raw_heat_pump_log_dto import RawHeatPumpLogDto
from weheat.models.read_all_heat_pump_dto import ReadAllHeatPumpDto
from weheat.models.read_heat_pump_dto import ReadHeatPumpDto
from weheat.models.read_user_dto import ReadUserDto
from weheat.models.read_user_me_dto import ReadUserMeDto
from weheat.models.role import Role



# coding: utf-8

"""
    Weheat Backend

    This is the backend for the Weheat project

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from typing import Any, ClassVar, Dict, List, Optional, Union
from pydantic import BaseModel, StrictFloat, StrictInt, StrictStr
from pydantic import Field
try:
    from typing import Self
except ImportError:
    from typing_extensions import Self

class HeatPumpLogViewDto(BaseModel):
    """
    HeatPumpLogViewDto
    """ # noqa: E501
    time_bucket: Optional[datetime] = Field(default=None, description="Timestamp of the beginning of this interval (UTC)", alias="timeBucket")
    interval: Optional[StrictStr] = Field(default=None, description="Interval Granularity of this HeatPumpLogViewDto (Correct intervals include: \"Hour\", \"Day\", \"Week\", \"Month\", \"Year\")")
    time_covered_in_interval: Optional[StrictInt] = Field(default=None, description="Amount of time covered by this interval in seconds out of a maximum of the Interval (Granularity) in seconds", alias="timeCoveredInInterval")
    heat_pump_state_standby: Optional[StrictInt] = Field(default=None, description="Amount of seconds the heat pump spend in the standby state for this interval", alias="heatPumpStateStandby")
    heat_pump_state_heating: Optional[StrictInt] = Field(default=None, description="Amount of seconds the heat pump spend in the heating state for this interval", alias="heatPumpStateHeating")
    heat_pump_state_defrost: Optional[StrictInt] = Field(default=None, description="Amount of seconds the heat pump spend in the defrost required active state for this interval", alias="heatPumpStateDefrost")
    heat_pump_state_cooling: Optional[StrictInt] = Field(default=None, description="Amount of seconds the heat pump spend in the cooling state for this interval", alias="heatPumpStateCooling")
    heat_pump_state_dhw: Optional[StrictInt] = Field(default=None, description="Amount of seconds the heat pump spend in the (heating) Domestic Hot Water stae for this interval", alias="heatPumpStateDHW")
    heat_pump_state_legionella: Optional[StrictInt] = Field(default=None, description="Amount of seconds the heat pump spend in the Legionnaires' Disease Prevention state for this interval", alias="heatPumpStateLegionella")
    rpm_control_startup: Optional[StrictInt] = Field(default=None, description="Amount of seconds the heat pump spend in one of the RPM Control Startup states for this interval", alias="rpmControlStartup")
    rpm_control_running: Optional[StrictInt] = Field(default=None, description="Amount of seconds the heat pump spend in one of the RPM Control Running state for this interval", alias="rpmControlRunning")
    rpm_control_running_abnormally: Optional[StrictInt] = Field(default=None, description="Amount of seconds the heat pump spend in the RPM Control Running Abnormally states for this interval", alias="rpmControlRunningAbnormally")
    rpm_control_shutdown: Optional[StrictInt] = Field(default=None, description="Amount of seconds the heat pump spend in one of the RPM Control Shutdown states for this interval", alias="rpmControlShutdown")
    control_bridge_status_water_pump: Optional[StrictInt] = Field(default=None, description="Amount of seconds the heat pump spend in the Control Bridge Status Water Pump state to true for this interval", alias="controlBridgeStatusWaterPump")
    control_bridge_status_dhw_valve: Optional[StrictInt] = Field(default=None, description="Amount of seconds the heat pump spend in the Control Bridge Status DHW Valve state to true for this interval", alias="controlBridgeStatusDhwValve")
    control_bridge_status_gas_boiler: Optional[StrictInt] = Field(default=None, description="Amount of seconds the heat pump spend in the Control Bridge Status Gas Boiler state to true for this interval", alias="controlBridgeStatusGasBoiler")
    control_bridge_status_electric_heater: Optional[StrictInt] = Field(default=None, description="Amount of seconds the heat pump spend in the Control Bridge Status Electric Heater state to true for this interval", alias="controlBridgeStatusElectricHeater")
    control_bridge_status_water_pump2: Optional[StrictInt] = Field(default=None, description="Amount of seconds the heat pump spend in the Control Bridge Status Water Pump 2 state to true for this interval", alias="controlBridgeStatusWaterPump2")
    dtc_none: Optional[StrictInt] = Field(default=None, description="Amount of seconds the heat pump spend in the No DTC codes Error state for this interval (staff only)", alias="dtcNone")
    dtc_continue: Optional[StrictInt] = Field(default=None, description="Amount of seconds the heat pump spend in the DTC but can Continue Error state for this interval (staff only)", alias="dtcContinue")
    dtc_compressor_off: Optional[StrictInt] = Field(default=None, description="Amount of seconds the heat pump spend in the DTC Compressor Off Error state for this interval (staff only)", alias="dtcCompressorOff")
    dtc_defrost_forbidden: Optional[StrictInt] = Field(default=None, description="Amount of seconds the heat pump spend in the DTC Defrost Forbidden Error state for this interval (staff only)", alias="dtcDefrostForbidden")
    dtc_request_service: Optional[StrictInt] = Field(default=None, description="Amount of seconds the heat pump spend in the DTC Request Service Error state for this interval (staff only)", alias="dtcRequestService")
    dtc_use_heating_curve: Optional[StrictInt] = Field(default=None, description="Amount of seconds the heat pump spend in the DTC Use Heating Curve (only) Error state for this interval (staff only)", alias="dtcUseHeatingCurve")
    dtc_dhw_forbidden: Optional[StrictInt] = Field(default=None, description="Amount of seconds the heat pump spend in the DTC HW Forbidden Error state for this interval (staff only)", alias="dtcDhwForbidden")
    dtc_error: Optional[StrictInt] = Field(default=None, description="Amount of seconds the heat pump spend in the DTC DTC Error state for this interval (staff only)", alias="dtcError")
    dtc_inactive: Optional[StrictInt] = Field(default=None, description="Amount of seconds the heat pump spend in the DTC Inactive Error state for this interval (staff only)", alias="dtcInactive")
    signal_strength_average: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Average of the Signal Strength (integer out of 20) of the heat pump for this interval (staff only)", alias="signalStrengthAverage")
    signal_strength_min: Optional[StrictInt] = Field(default=None, description="Minimal Signal Strength (integer out of 20) of the heat pump for this interval (staff only)", alias="signalStrengthMin")
    signal_strength_max: Optional[StrictInt] = Field(default=None, description="Maximum Signal Strength (integer out of 20) of the heat pump for this interval (staff only)", alias="signalStrengthMax")
    t1_average: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Average temperature of the top DHW sensor (T1) of the heat pump for this interval", alias="t1Average")
    t1_min: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Minimum temperature of the top DHW sensor (T1) of the heat pump for this interval", alias="t1Min")
    t1_max: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum temperature of the top DHW sensor (T1) of the heat pump for this interval", alias="t1Max")
    t2_average: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Average temperature of the bottom DHW sensor (T2) of the heat pump for this interval", alias="t2Average")
    t2_min: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Minimum temperature of the bottom DHW sensor (T2) of the heat pump for this interval", alias="t2Min")
    t2_max: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum temperature of the bottom DHW sensor (T2) of the heat pump for this interval", alias="t2Max")
    t_spare_ntc_average: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Average temperature of the spare DHW sensor (TSpareNTC) of the heat pump for this interval (staff only)", alias="tSpareNtcAverage")
    t_spare_ntc_min: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Minimum temperature of the spare DHW sensor (TSpareNTC) of the heat pump for this interval (staff only)", alias="tSpareNtcMin")
    t_spare_ntc_max: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum temperature of the spare DHW sensor (TSpareNTC) of the heat pump for this interval (staff only)", alias="tSpareNtcMax")
    t_board_average: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Average temperature of the control board of the heat pump for this interval (staff only)", alias="tBoardAverage")
    t_board_min: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Minimum temperature of the control board of the heat pump for this interval (staff only)", alias="tBoardMin")
    t_board_max: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum temperature of the control board of the heat pump for this interval (staff only)", alias="tBoardMax")
    t_air_in_average: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Average temperature of air going into the heat pump for this interval", alias="tAirInAverage")
    t_air_in_min: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, alias="tAirInMin")
    t_air_in_max: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, alias="tAirInMax")
    t_air_out_average: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Average temperature of air going out of the heat pump for this interval", alias="tAirOutAverage")
    t_air_out_min: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Minimum temperature of air going out of the heat pump for this interval", alias="tAirOutMin")
    t_air_out_max: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum temperature of air going out of the heat pump for this interval", alias="tAirOutMax")
    t_water_in_average: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Average temperature of the water going into the heat pump for this interval", alias="tWaterInAverage")
    t_water_in_min: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Minimum temperature of the water going into the heat pump for this interval", alias="tWaterInMin")
    t_water_in_max: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum temperature of the water going into the heat pump for this interval", alias="tWaterInMax")
    t_water_out_average: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Average temperature of the water going out of the heat pump for this interval", alias="tWaterOutAverage")
    t_water_out_min: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Minimum temperature of the water going out of the heat pump for this interval", alias="tWaterOutMin")
    t_water_out_max: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum temperature of the water going out of the heat pump for this interval", alias="tWaterOutMax")
    t_water_house_in_average: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Average temperature of the water going into the house for this interval", alias="tWaterHouseInAverage")
    t_water_house_in_min: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Minimum temperature of the water going into the house for this interval", alias="tWaterHouseInMin")
    t_water_house_in_max: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum temperature of the water going into the house for this interval", alias="tWaterHouseInMax")
    rpm_average: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Average RPM of the compressor for this interval (not for consumers)", alias="rpmAverage")
    rpm_min: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Minimum RPM of the compressor for this interval (not for consumers)", alias="rpmMin")
    rpm_max: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum RPM of the compressor for this interval (not for consumers)", alias="rpmMax")
    rpm_limiter_average: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Average RPM value the heat pump was limited to for this interval (staff only)", alias="rpmLimiterAverage")
    rpm_limiter_min: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Minimum RPM value the heat pump was limited to for this interval (staff only)", alias="rpmLimiterMin")
    rpm_limiter_max: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum RPM value the heat pump was limited to for this interval (staff only)", alias="rpmLimiterMax")
    rpm_limiter_no_limit: Optional[StrictInt] = Field(default=None, description="Amount of time the compressor RPM was not Limited at all by the RPM Limiter state for this interval (staff only)", alias="rpmLimiterNoLimit")
    rpm_limiter_power_limit: Optional[StrictInt] = Field(default=None, description="Amount of time the compressor RPM was being most limited by the amount of Power by the RPM Limiter state for this interval (staff only)", alias="rpmLimiterPowerLimit")
    rpm_limiter_defrost: Optional[StrictInt] = Field(default=None, description="Amount of time the compressor RPM was being most limited because of Defrost by the RPM Limiter state for this interval (staff only)", alias="rpmLimiterDefrost")
    rpm_limiter_silent_hours: Optional[StrictInt] = Field(default=None, description="Amount of time the compressor RPM was being most limited because of Silent Hours by the RPM Limiter state for this interval (staff only)", alias="rpmLimiterSilentHours")
    rpm_limiter_hp_control: Optional[StrictInt] = Field(default=None, description="Amount of time the compressor RPM was being most limited because of HP Control method by the RPM Limiter state for this interval (staff only)", alias="rpmLimiterHPControl")
    rpm_limiter_pressure: Optional[StrictInt] = Field(default=None, description="Amount of time the compressor RPM was being most limited because of its Pressure by the RPM Limiter state for this interval (staff only)", alias="rpmLimiterPressure")
    rpm_limiter_water_out: Optional[StrictInt] = Field(default=None, description="Amount of time the compressor RPM was being most limited because of its Water Out temperature by the RPM Limiter state for this interval (staff only)", alias="rpmLimiterWaterOut")
    rpm_limiter_envelope: Optional[StrictInt] = Field(default=None, description="Amount of time the compressor RPM was being most limited because of Envelope control method by the RPM Limiter state for this interval (staff only)", alias="rpmLimiterEnvelope")
    rpm_limiter_house_in: Optional[StrictInt] = Field(default=None, description="Amount of time the compressor RPM was being most limited because of its Water In temperature by the RPM Limiter state for this interval (staff only)", alias="rpmLimiterHouseIn")
    p_compressor_in_average: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Average pressure on the intake side of the compressor for this interval (not for consumers)", alias="pCompressorInAverage")
    p_compressor_in_min: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Minimum pressure on the intake side of the compressor for this interval (not for consumers)", alias="pCompressorInMin")
    p_compressor_in_max: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum pressure on the intake side of the compressor for this interval (not for consumers)", alias="pCompressorInMax")
    p_compressor_out_average: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Average pressure on the output side of the compressor for this interval (not for consumers)", alias="pCompressorOutAverage")
    p_compressor_out_min: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Minimum pressure on the output side of the compressor for this interval (not for consumers)", alias="pCompressorOutMin")
    p_compressor_out_max: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum pressure on the output side of the compressor for this interval (not for consumers)", alias="pCompressorOutMax")
    p_compressor_in_target_average: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Average target pressure (on the intake side) of the compressor for this interval (not for consumers)", alias="pCompressorInTargetAverage")
    p_compressor_in_target_min: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Minimum target pressure (on the intake side) of the compressor for this interval (not for consumers)", alias="pCompressorInTargetMin")
    p_compressor_in_target_max: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum target pressure (on the intake side) of the compressor for this interval (not for consumers)", alias="pCompressorInTargetMax")
    t_inverter_average: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Average temperature of the inverter for this interval (staff only)", alias="tInverterAverage")
    t_inverter_min: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Minimum temperature of the inverter for this interval (staff only)", alias="tInverterMin")
    t_inverter_max: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum temperature of the inverter for this interval (staff only)", alias="tInverterMax")
    t_compressor_in_average: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Average temperature of intake side of the compressor for this interval (not for consumers)", alias="tCompressorInAverage")
    t_compressor_in_min: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Minimum temperature of intake side of the compressor for this interval (not for consumers)", alias="tCompressorInMin")
    t_compressor_in_max: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum temperature of intake side of the compressor for this interval (not for consumers)", alias="tCompressorInMax")
    t_compressor_out_average: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Average temperature of output side of the compressor for this interval (not for consumers)", alias="tCompressorOutAverage")
    t_compressor_out_min: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Minimum temperature of output side of the compressor for this interval (not for consumers)", alias="tCompressorOutMin")
    t_compressor_out_max: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum temperature of output side of the compressor for this interval (not for consumers)", alias="tCompressorOutMax")
    t_compressor_in_transient_average: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Average temperature transient for the intake side of the compressor for this interval (staff only)", alias="tCompressorInTransientAverage")
    t_compressor_in_transient_min: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Minimum temperature transient for the intake side of the compressor for this interval (staff only)", alias="tCompressorInTransientMin")
    t_compressor_in_transient_max: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum temperature transient for the intake side of the compressor for this interval (staff only)", alias="tCompressorInTransientMax")
    t_compressor_out_transient_average: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Average temperature transient for the output side of the compressor for this interval (staff only)", alias="tCompressorOutTransientAverage")
    t_compressor_out_transient_min: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Minimum temperature transient for the output side of the compressor for this interval (staff only)", alias="tCompressorOutTransientMin")
    t_compressor_out_transient_max: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum temperature transient for the output side of the compressor for this interval (staff only)", alias="tCompressorOutTransientMax")
    delta_t_compressor_in_superheat_average: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Average temperature difference in the compressor for Superheat for this interval (staff only)", alias="deltaTCompressorInSuperheatAverage")
    delta_t_compressor_in_superheat_min: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Minimum temperature difference in the compressor for Superheat for this interval (staff only)", alias="deltaTCompressorInSuperheatMin")
    delta_t_compressor_in_superheat_max: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum temperature difference in the compressor for Superheat for this interval (staff only)", alias="deltaTCompressorInSuperheatMax")
    compressor_power_low_accuracy_average: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Average power used by the compressor (low accuracy) for this interval (staff only)", alias="compressorPowerLowAccuracyAverage")
    compressor_power_low_accuracy_min: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Minimum power used by the compressor (low accuracy) for this interval (staff only)", alias="compressorPowerLowAccuracyMin")
    compressor_power_low_accuracy_max: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum power used by the compressor (low accuracy) for this interval (staff only)", alias="compressorPowerLowAccuracyMax")
    fan_average: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Average percentage of the fan used for this interval (staff only)", alias="fanAverage")
    fan_min: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Minimum percentage of the fan used for this interval (staff only)", alias="fanMin")
    fan_max: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum percentage of the fan used for this interval (staff only)", alias="fanMax")
    fan_power_average: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Average power used by the fan for this interval (staff only)", alias="fanPowerAverage")
    fan_power_min: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Minimum power used by the fan for this interval (staff only)", alias="fanPowerMin")
    fan_power_max: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum power used by the fan for this interval (staff only)", alias="fanPowerMax")
    cm_mass_power_in_average: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Average of the total amount of energy produced by the heat pump for this interval (staff only)", alias="cmMassPowerInAverage")
    cm_mass_power_out_average: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Average of the total amount of energy produced by the heat pump for this interval (staff only)", alias="cmMassPowerOutAverage")
    temperature_error_integral_average: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Average value of the TemperatureErrorIntegral for this interval (staff only)", alias="temperatureErrorIntegralAverage")
    temperature_error_integral_min: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Minimum value of the TemperatureErrorIntegral for this interval (staff only)", alias="temperatureErrorIntegralMin")
    temperature_error_integral_max: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum value of the TemperatureErrorIntegral for this interval (staff only)", alias="temperatureErrorIntegralMax")
    thermostat_state_off: Optional[StrictInt] = Field(default=None, description="Amount of seconds the heat pump spend in the Thermostat State Off state for this interval", alias="thermostatStateOff")
    thermostat_state_on: Optional[StrictInt] = Field(default=None, description="Amount of seconds the heat pump spend in the Thermostat State On state for this interval", alias="thermostatStateOn")
    t_room_average: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Average temperature of the room for this interval", alias="tRoomAverage")
    t_room_min: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Minimum temperature of the room for this interval", alias="tRoomMin")
    t_room_max: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum temperature of the room for this interval", alias="tRoomMax")
    t_room_target_average: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Average target room temperature of the thermostat for this interval", alias="tRoomTargetAverage")
    t_room_target_min: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Minimum target room temperature of the thermostat for this interval", alias="tRoomTargetMin")
    t_room_target_max: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum target room temperature of the thermostat for this interval", alias="tRoomTargetMax")
    t_thermostat_setpoint_average: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Average temperature setpoint of the thermostat for this interval OR the water setpoint of the heat pump", alias="tThermostatSetpointAverage")
    t_thermostat_setpoint_min: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Minimum temperature setpoint of the thermostat for this interval OR the water setpoint of the heat pump", alias="tThermostatSetpointMin")
    t_thermostat_setpoint_max: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum temperature setpoint of the thermostat for this interval OR the water setpoint of the heat pump", alias="tThermostatSetpointMax")
    ot_boiler_feed_temperature_average: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Average OpenTherm boiler water feed temperature for this interval", alias="otBoilerFeedTemperatureAverage")
    ot_boiler_feed_temperature_min: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Minimum OpenTherm boiler water feed temperature for this interval", alias="otBoilerFeedTemperatureMin")
    ot_boiler_feed_temperature_max: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum OpenTherm boiler water feed temperature for this interval", alias="otBoilerFeedTemperatureMax")
    ot_boiler_return_temperature_average: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Average OpenTherm boiler water return temperature for this interval", alias="otBoilerReturnTemperatureAverage")
    ot_boiler_return_temperature_min: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Minimum OpenTherm boiler water return temperature for this interval", alias="otBoilerReturnTemperatureMin")
    ot_boiler_return_temperature_max: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Maximum OpenTherm boiler water return temperature for this interval", alias="otBoilerReturnTemperatureMax")
    __properties: ClassVar[List[str]] = ["timeBucket", "interval", "timeCoveredInInterval", "heatPumpStateStandby", "heatPumpStateHeating", "heatPumpStateDefrost", "heatPumpStateCooling", "heatPumpStateDHW", "heatPumpStateLegionella", "rpmControlStartup", "rpmControlRunning", "rpmControlRunningAbnormally", "rpmControlShutdown", "controlBridgeStatusWaterPump", "controlBridgeStatusDhwValve", "controlBridgeStatusGasBoiler", "controlBridgeStatusElectricHeater", "controlBridgeStatusWaterPump2", "dtcNone", "dtcContinue", "dtcCompressorOff", "dtcDefrostForbidden", "dtcRequestService", "dtcUseHeatingCurve", "dtcDhwForbidden", "dtcError", "dtcInactive", "signalStrengthAverage", "signalStrengthMin", "signalStrengthMax", "t1Average", "t1Min", "t1Max", "t2Average", "t2Min", "t2Max", "tSpareNtcAverage", "tSpareNtcMin", "tSpareNtcMax", "tBoardAverage", "tBoardMin", "tBoardMax", "tAirInAverage", "tAirInMin", "tAirInMax", "tAirOutAverage", "tAirOutMin", "tAirOutMax", "tWaterInAverage", "tWaterInMin", "tWaterInMax", "tWaterOutAverage", "tWaterOutMin", "tWaterOutMax", "tWaterHouseInAverage", "tWaterHouseInMin", "tWaterHouseInMax", "rpmAverage", "rpmMin", "rpmMax", "rpmLimiterAverage", "rpmLimiterMin", "rpmLimiterMax", "rpmLimiterNoLimit", "rpmLimiterPowerLimit", "rpmLimiterDefrost", "rpmLimiterSilentHours", "rpmLimiterHPControl", "rpmLimiterPressure", "rpmLimiterWaterOut", "rpmLimiterEnvelope", "rpmLimiterHouseIn", "pCompressorInAverage", "pCompressorInMin", "pCompressorInMax", "pCompressorOutAverage", "pCompressorOutMin", "pCompressorOutMax", "pCompressorInTargetAverage", "pCompressorInTargetMin", "pCompressorInTargetMax", "tInverterAverage", "tInverterMin", "tInverterMax", "tCompressorInAverage", "tCompressorInMin", "tCompressorInMax", "tCompressorOutAverage", "tCompressorOutMin", "tCompressorOutMax", "tCompressorInTransientAverage", "tCompressorInTransientMin", "tCompressorInTransientMax", "tCompressorOutTransientAverage", "tCompressorOutTransientMin", "tCompressorOutTransientMax", "deltaTCompressorInSuperheatAverage", "deltaTCompressorInSuperheatMin", "deltaTCompressorInSuperheatMax", "compressorPowerLowAccuracyAverage", "compressorPowerLowAccuracyMin", "compressorPowerLowAccuracyMax", "fanAverage", "fanMin", "fanMax", "fanPowerAverage", "fanPowerMin", "fanPowerMax", "cmMassPowerInAverage", "cmMassPowerOutAverage", "temperatureErrorIntegralAverage", "temperatureErrorIntegralMin", "temperatureErrorIntegralMax", "thermostatStateOff", "thermostatStateOn", "tRoomAverage", "tRoomMin", "tRoomMax", "tRoomTargetAverage", "tRoomTargetMin", "tRoomTargetMax", "tThermostatSetpointAverage", "tThermostatSetpointMin", "tThermostatSetpointMax", "otBoilerFeedTemperatureAverage", "otBoilerFeedTemperatureMin", "otBoilerFeedTemperatureMax", "otBoilerReturnTemperatureAverage", "otBoilerReturnTemperatureMin", "otBoilerReturnTemperatureMax"]

    model_config = {
        "populate_by_name": True,
        "validate_assignment": True,
        "protected_namespaces": (),
    }


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of HeatPumpLogViewDto from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={
            },
            exclude_none=True,
        )
        # set to None if interval (nullable) is None
        # and model_fields_set contains the field
        if self.interval is None and "interval" in self.model_fields_set:
            _dict['interval'] = None

        # set to None if rpm_control_startup (nullable) is None
        # and model_fields_set contains the field
        if self.rpm_control_startup is None and "rpm_control_startup" in self.model_fields_set:
            _dict['rpmControlStartup'] = None

        # set to None if rpm_control_running (nullable) is None
        # and model_fields_set contains the field
        if self.rpm_control_running is None and "rpm_control_running" in self.model_fields_set:
            _dict['rpmControlRunning'] = None

        # set to None if rpm_control_shutdown (nullable) is None
        # and model_fields_set contains the field
        if self.rpm_control_shutdown is None and "rpm_control_shutdown" in self.model_fields_set:
            _dict['rpmControlShutdown'] = None

        # set to None if control_bridge_status_dhw_valve (nullable) is None
        # and model_fields_set contains the field
        if self.control_bridge_status_dhw_valve is None and "control_bridge_status_dhw_valve" in self.model_fields_set:
            _dict['controlBridgeStatusDhwValve'] = None

        # set to None if dtc_none (nullable) is None
        # and model_fields_set contains the field
        if self.dtc_none is None and "dtc_none" in self.model_fields_set:
            _dict['dtcNone'] = None

        # set to None if dtc_continue (nullable) is None
        # and model_fields_set contains the field
        if self.dtc_continue is None and "dtc_continue" in self.model_fields_set:
            _dict['dtcContinue'] = None

        # set to None if dtc_compressor_off (nullable) is None
        # and model_fields_set contains the field
        if self.dtc_compressor_off is None and "dtc_compressor_off" in self.model_fields_set:
            _dict['dtcCompressorOff'] = None

        # set to None if dtc_defrost_forbidden (nullable) is None
        # and model_fields_set contains the field
        if self.dtc_defrost_forbidden is None and "dtc_defrost_forbidden" in self.model_fields_set:
            _dict['dtcDefrostForbidden'] = None

        # set to None if dtc_request_service (nullable) is None
        # and model_fields_set contains the field
        if self.dtc_request_service is None and "dtc_request_service" in self.model_fields_set:
            _dict['dtcRequestService'] = None

        # set to None if dtc_use_heating_curve (nullable) is None
        # and model_fields_set contains the field
        if self.dtc_use_heating_curve is None and "dtc_use_heating_curve" in self.model_fields_set:
            _dict['dtcUseHeatingCurve'] = None

        # set to None if dtc_dhw_forbidden (nullable) is None
        # and model_fields_set contains the field
        if self.dtc_dhw_forbidden is None and "dtc_dhw_forbidden" in self.model_fields_set:
            _dict['dtcDhwForbidden'] = None

        # set to None if dtc_error (nullable) is None
        # and model_fields_set contains the field
        if self.dtc_error is None and "dtc_error" in self.model_fields_set:
            _dict['dtcError'] = None

        # set to None if dtc_inactive (nullable) is None
        # and model_fields_set contains the field
        if self.dtc_inactive is None and "dtc_inactive" in self.model_fields_set:
            _dict['dtcInactive'] = None

        # set to None if signal_strength_average (nullable) is None
        # and model_fields_set contains the field
        if self.signal_strength_average is None and "signal_strength_average" in self.model_fields_set:
            _dict['signalStrengthAverage'] = None

        # set to None if signal_strength_min (nullable) is None
        # and model_fields_set contains the field
        if self.signal_strength_min is None and "signal_strength_min" in self.model_fields_set:
            _dict['signalStrengthMin'] = None

        # set to None if signal_strength_max (nullable) is None
        # and model_fields_set contains the field
        if self.signal_strength_max is None and "signal_strength_max" in self.model_fields_set:
            _dict['signalStrengthMax'] = None

        # set to None if t1_average (nullable) is None
        # and model_fields_set contains the field
        if self.t1_average is None and "t1_average" in self.model_fields_set:
            _dict['t1Average'] = None

        # set to None if t1_min (nullable) is None
        # and model_fields_set contains the field
        if self.t1_min is None and "t1_min" in self.model_fields_set:
            _dict['t1Min'] = None

        # set to None if t1_max (nullable) is None
        # and model_fields_set contains the field
        if self.t1_max is None and "t1_max" in self.model_fields_set:
            _dict['t1Max'] = None

        # set to None if t2_average (nullable) is None
        # and model_fields_set contains the field
        if self.t2_average is None and "t2_average" in self.model_fields_set:
            _dict['t2Average'] = None

        # set to None if t2_min (nullable) is None
        # and model_fields_set contains the field
        if self.t2_min is None and "t2_min" in self.model_fields_set:
            _dict['t2Min'] = None

        # set to None if t2_max (nullable) is None
        # and model_fields_set contains the field
        if self.t2_max is None and "t2_max" in self.model_fields_set:
            _dict['t2Max'] = None

        # set to None if t_spare_ntc_average (nullable) is None
        # and model_fields_set contains the field
        if self.t_spare_ntc_average is None and "t_spare_ntc_average" in self.model_fields_set:
            _dict['tSpareNtcAverage'] = None

        # set to None if t_spare_ntc_min (nullable) is None
        # and model_fields_set contains the field
        if self.t_spare_ntc_min is None and "t_spare_ntc_min" in self.model_fields_set:
            _dict['tSpareNtcMin'] = None

        # set to None if t_spare_ntc_max (nullable) is None
        # and model_fields_set contains the field
        if self.t_spare_ntc_max is None and "t_spare_ntc_max" in self.model_fields_set:
            _dict['tSpareNtcMax'] = None

        # set to None if t_board_average (nullable) is None
        # and model_fields_set contains the field
        if self.t_board_average is None and "t_board_average" in self.model_fields_set:
            _dict['tBoardAverage'] = None

        # set to None if t_board_min (nullable) is None
        # and model_fields_set contains the field
        if self.t_board_min is None and "t_board_min" in self.model_fields_set:
            _dict['tBoardMin'] = None

        # set to None if t_board_max (nullable) is None
        # and model_fields_set contains the field
        if self.t_board_max is None and "t_board_max" in self.model_fields_set:
            _dict['tBoardMax'] = None

        # set to None if t_air_out_average (nullable) is None
        # and model_fields_set contains the field
        if self.t_air_out_average is None and "t_air_out_average" in self.model_fields_set:
            _dict['tAirOutAverage'] = None

        # set to None if t_air_out_min (nullable) is None
        # and model_fields_set contains the field
        if self.t_air_out_min is None and "t_air_out_min" in self.model_fields_set:
            _dict['tAirOutMin'] = None

        # set to None if t_air_out_max (nullable) is None
        # and model_fields_set contains the field
        if self.t_air_out_max is None and "t_air_out_max" in self.model_fields_set:
            _dict['tAirOutMax'] = None

        # set to None if rpm_average (nullable) is None
        # and model_fields_set contains the field
        if self.rpm_average is None and "rpm_average" in self.model_fields_set:
            _dict['rpmAverage'] = None

        # set to None if rpm_min (nullable) is None
        # and model_fields_set contains the field
        if self.rpm_min is None and "rpm_min" in self.model_fields_set:
            _dict['rpmMin'] = None

        # set to None if rpm_max (nullable) is None
        # and model_fields_set contains the field
        if self.rpm_max is None and "rpm_max" in self.model_fields_set:
            _dict['rpmMax'] = None

        # set to None if rpm_limiter_average (nullable) is None
        # and model_fields_set contains the field
        if self.rpm_limiter_average is None and "rpm_limiter_average" in self.model_fields_set:
            _dict['rpmLimiterAverage'] = None

        # set to None if rpm_limiter_min (nullable) is None
        # and model_fields_set contains the field
        if self.rpm_limiter_min is None and "rpm_limiter_min" in self.model_fields_set:
            _dict['rpmLimiterMin'] = None

        # set to None if rpm_limiter_max (nullable) is None
        # and model_fields_set contains the field
        if self.rpm_limiter_max is None and "rpm_limiter_max" in self.model_fields_set:
            _dict['rpmLimiterMax'] = None

        # set to None if rpm_limiter_no_limit (nullable) is None
        # and model_fields_set contains the field
        if self.rpm_limiter_no_limit is None and "rpm_limiter_no_limit" in self.model_fields_set:
            _dict['rpmLimiterNoLimit'] = None

        # set to None if rpm_limiter_power_limit (nullable) is None
        # and model_fields_set contains the field
        if self.rpm_limiter_power_limit is None and "rpm_limiter_power_limit" in self.model_fields_set:
            _dict['rpmLimiterPowerLimit'] = None

        # set to None if rpm_limiter_defrost (nullable) is None
        # and model_fields_set contains the field
        if self.rpm_limiter_defrost is None and "rpm_limiter_defrost" in self.model_fields_set:
            _dict['rpmLimiterDefrost'] = None

        # set to None if rpm_limiter_silent_hours (nullable) is None
        # and model_fields_set contains the field
        if self.rpm_limiter_silent_hours is None and "rpm_limiter_silent_hours" in self.model_fields_set:
            _dict['rpmLimiterSilentHours'] = None

        # set to None if rpm_limiter_hp_control (nullable) is None
        # and model_fields_set contains the field
        if self.rpm_limiter_hp_control is None and "rpm_limiter_hp_control" in self.model_fields_set:
            _dict['rpmLimiterHPControl'] = None

        # set to None if rpm_limiter_pressure (nullable) is None
        # and model_fields_set contains the field
        if self.rpm_limiter_pressure is None and "rpm_limiter_pressure" in self.model_fields_set:
            _dict['rpmLimiterPressure'] = None

        # set to None if rpm_limiter_water_out (nullable) is None
        # and model_fields_set contains the field
        if self.rpm_limiter_water_out is None and "rpm_limiter_water_out" in self.model_fields_set:
            _dict['rpmLimiterWaterOut'] = None

        # set to None if rpm_limiter_envelope (nullable) is None
        # and model_fields_set contains the field
        if self.rpm_limiter_envelope is None and "rpm_limiter_envelope" in self.model_fields_set:
            _dict['rpmLimiterEnvelope'] = None

        # set to None if rpm_limiter_house_in (nullable) is None
        # and model_fields_set contains the field
        if self.rpm_limiter_house_in is None and "rpm_limiter_house_in" in self.model_fields_set:
            _dict['rpmLimiterHouseIn'] = None

        # set to None if p_compressor_in_average (nullable) is None
        # and model_fields_set contains the field
        if self.p_compressor_in_average is None and "p_compressor_in_average" in self.model_fields_set:
            _dict['pCompressorInAverage'] = None

        # set to None if p_compressor_in_min (nullable) is None
        # and model_fields_set contains the field
        if self.p_compressor_in_min is None and "p_compressor_in_min" in self.model_fields_set:
            _dict['pCompressorInMin'] = None

        # set to None if p_compressor_in_max (nullable) is None
        # and model_fields_set contains the field
        if self.p_compressor_in_max is None and "p_compressor_in_max" in self.model_fields_set:
            _dict['pCompressorInMax'] = None

        # set to None if p_compressor_out_average (nullable) is None
        # and model_fields_set contains the field
        if self.p_compressor_out_average is None and "p_compressor_out_average" in self.model_fields_set:
            _dict['pCompressorOutAverage'] = None

        # set to None if p_compressor_out_min (nullable) is None
        # and model_fields_set contains the field
        if self.p_compressor_out_min is None and "p_compressor_out_min" in self.model_fields_set:
            _dict['pCompressorOutMin'] = None

        # set to None if p_compressor_out_max (nullable) is None
        # and model_fields_set contains the field
        if self.p_compressor_out_max is None and "p_compressor_out_max" in self.model_fields_set:
            _dict['pCompressorOutMax'] = None

        # set to None if p_compressor_in_target_average (nullable) is None
        # and model_fields_set contains the field
        if self.p_compressor_in_target_average is None and "p_compressor_in_target_average" in self.model_fields_set:
            _dict['pCompressorInTargetAverage'] = None

        # set to None if p_compressor_in_target_min (nullable) is None
        # and model_fields_set contains the field
        if self.p_compressor_in_target_min is None and "p_compressor_in_target_min" in self.model_fields_set:
            _dict['pCompressorInTargetMin'] = None

        # set to None if p_compressor_in_target_max (nullable) is None
        # and model_fields_set contains the field
        if self.p_compressor_in_target_max is None and "p_compressor_in_target_max" in self.model_fields_set:
            _dict['pCompressorInTargetMax'] = None

        # set to None if t_inverter_average (nullable) is None
        # and model_fields_set contains the field
        if self.t_inverter_average is None and "t_inverter_average" in self.model_fields_set:
            _dict['tInverterAverage'] = None

        # set to None if t_inverter_min (nullable) is None
        # and model_fields_set contains the field
        if self.t_inverter_min is None and "t_inverter_min" in self.model_fields_set:
            _dict['tInverterMin'] = None

        # set to None if t_inverter_max (nullable) is None
        # and model_fields_set contains the field
        if self.t_inverter_max is None and "t_inverter_max" in self.model_fields_set:
            _dict['tInverterMax'] = None

        # set to None if t_compressor_in_average (nullable) is None
        # and model_fields_set contains the field
        if self.t_compressor_in_average is None and "t_compressor_in_average" in self.model_fields_set:
            _dict['tCompressorInAverage'] = None

        # set to None if t_compressor_in_min (nullable) is None
        # and model_fields_set contains the field
        if self.t_compressor_in_min is None and "t_compressor_in_min" in self.model_fields_set:
            _dict['tCompressorInMin'] = None

        # set to None if t_compressor_in_max (nullable) is None
        # and model_fields_set contains the field
        if self.t_compressor_in_max is None and "t_compressor_in_max" in self.model_fields_set:
            _dict['tCompressorInMax'] = None

        # set to None if t_compressor_out_average (nullable) is None
        # and model_fields_set contains the field
        if self.t_compressor_out_average is None and "t_compressor_out_average" in self.model_fields_set:
            _dict['tCompressorOutAverage'] = None

        # set to None if t_compressor_out_min (nullable) is None
        # and model_fields_set contains the field
        if self.t_compressor_out_min is None and "t_compressor_out_min" in self.model_fields_set:
            _dict['tCompressorOutMin'] = None

        # set to None if t_compressor_out_max (nullable) is None
        # and model_fields_set contains the field
        if self.t_compressor_out_max is None and "t_compressor_out_max" in self.model_fields_set:
            _dict['tCompressorOutMax'] = None

        # set to None if t_compressor_in_transient_average (nullable) is None
        # and model_fields_set contains the field
        if self.t_compressor_in_transient_average is None and "t_compressor_in_transient_average" in self.model_fields_set:
            _dict['tCompressorInTransientAverage'] = None

        # set to None if t_compressor_in_transient_min (nullable) is None
        # and model_fields_set contains the field
        if self.t_compressor_in_transient_min is None and "t_compressor_in_transient_min" in self.model_fields_set:
            _dict['tCompressorInTransientMin'] = None

        # set to None if t_compressor_in_transient_max (nullable) is None
        # and model_fields_set contains the field
        if self.t_compressor_in_transient_max is None and "t_compressor_in_transient_max" in self.model_fields_set:
            _dict['tCompressorInTransientMax'] = None

        # set to None if t_compressor_out_transient_average (nullable) is None
        # and model_fields_set contains the field
        if self.t_compressor_out_transient_average is None and "t_compressor_out_transient_average" in self.model_fields_set:
            _dict['tCompressorOutTransientAverage'] = None

        # set to None if t_compressor_out_transient_min (nullable) is None
        # and model_fields_set contains the field
        if self.t_compressor_out_transient_min is None and "t_compressor_out_transient_min" in self.model_fields_set:
            _dict['tCompressorOutTransientMin'] = None

        # set to None if t_compressor_out_transient_max (nullable) is None
        # and model_fields_set contains the field
        if self.t_compressor_out_transient_max is None and "t_compressor_out_transient_max" in self.model_fields_set:
            _dict['tCompressorOutTransientMax'] = None

        # set to None if delta_t_compressor_in_superheat_average (nullable) is None
        # and model_fields_set contains the field
        if self.delta_t_compressor_in_superheat_average is None and "delta_t_compressor_in_superheat_average" in self.model_fields_set:
            _dict['deltaTCompressorInSuperheatAverage'] = None

        # set to None if delta_t_compressor_in_superheat_min (nullable) is None
        # and model_fields_set contains the field
        if self.delta_t_compressor_in_superheat_min is None and "delta_t_compressor_in_superheat_min" in self.model_fields_set:
            _dict['deltaTCompressorInSuperheatMin'] = None

        # set to None if delta_t_compressor_in_superheat_max (nullable) is None
        # and model_fields_set contains the field
        if self.delta_t_compressor_in_superheat_max is None and "delta_t_compressor_in_superheat_max" in self.model_fields_set:
            _dict['deltaTCompressorInSuperheatMax'] = None

        # set to None if compressor_power_low_accuracy_average (nullable) is None
        # and model_fields_set contains the field
        if self.compressor_power_low_accuracy_average is None and "compressor_power_low_accuracy_average" in self.model_fields_set:
            _dict['compressorPowerLowAccuracyAverage'] = None

        # set to None if compressor_power_low_accuracy_min (nullable) is None
        # and model_fields_set contains the field
        if self.compressor_power_low_accuracy_min is None and "compressor_power_low_accuracy_min" in self.model_fields_set:
            _dict['compressorPowerLowAccuracyMin'] = None

        # set to None if compressor_power_low_accuracy_max (nullable) is None
        # and model_fields_set contains the field
        if self.compressor_power_low_accuracy_max is None and "compressor_power_low_accuracy_max" in self.model_fields_set:
            _dict['compressorPowerLowAccuracyMax'] = None

        # set to None if fan_average (nullable) is None
        # and model_fields_set contains the field
        if self.fan_average is None and "fan_average" in self.model_fields_set:
            _dict['fanAverage'] = None

        # set to None if fan_min (nullable) is None
        # and model_fields_set contains the field
        if self.fan_min is None and "fan_min" in self.model_fields_set:
            _dict['fanMin'] = None

        # set to None if fan_max (nullable) is None
        # and model_fields_set contains the field
        if self.fan_max is None and "fan_max" in self.model_fields_set:
            _dict['fanMax'] = None

        # set to None if fan_power_average (nullable) is None
        # and model_fields_set contains the field
        if self.fan_power_average is None and "fan_power_average" in self.model_fields_set:
            _dict['fanPowerAverage'] = None

        # set to None if fan_power_min (nullable) is None
        # and model_fields_set contains the field
        if self.fan_power_min is None and "fan_power_min" in self.model_fields_set:
            _dict['fanPowerMin'] = None

        # set to None if fan_power_max (nullable) is None
        # and model_fields_set contains the field
        if self.fan_power_max is None and "fan_power_max" in self.model_fields_set:
            _dict['fanPowerMax'] = None

        # set to None if cm_mass_power_in_average (nullable) is None
        # and model_fields_set contains the field
        if self.cm_mass_power_in_average is None and "cm_mass_power_in_average" in self.model_fields_set:
            _dict['cmMassPowerInAverage'] = None

        # set to None if cm_mass_power_out_average (nullable) is None
        # and model_fields_set contains the field
        if self.cm_mass_power_out_average is None and "cm_mass_power_out_average" in self.model_fields_set:
            _dict['cmMassPowerOutAverage'] = None

        # set to None if temperature_error_integral_average (nullable) is None
        # and model_fields_set contains the field
        if self.temperature_error_integral_average is None and "temperature_error_integral_average" in self.model_fields_set:
            _dict['temperatureErrorIntegralAverage'] = None

        # set to None if temperature_error_integral_min (nullable) is None
        # and model_fields_set contains the field
        if self.temperature_error_integral_min is None and "temperature_error_integral_min" in self.model_fields_set:
            _dict['temperatureErrorIntegralMin'] = None

        # set to None if temperature_error_integral_max (nullable) is None
        # and model_fields_set contains the field
        if self.temperature_error_integral_max is None and "temperature_error_integral_max" in self.model_fields_set:
            _dict['temperatureErrorIntegralMax'] = None

        # set to None if ot_boiler_feed_temperature_average (nullable) is None
        # and model_fields_set contains the field
        if self.ot_boiler_feed_temperature_average is None and "ot_boiler_feed_temperature_average" in self.model_fields_set:
            _dict['otBoilerFeedTemperatureAverage'] = None

        # set to None if ot_boiler_feed_temperature_min (nullable) is None
        # and model_fields_set contains the field
        if self.ot_boiler_feed_temperature_min is None and "ot_boiler_feed_temperature_min" in self.model_fields_set:
            _dict['otBoilerFeedTemperatureMin'] = None

        # set to None if ot_boiler_feed_temperature_max (nullable) is None
        # and model_fields_set contains the field
        if self.ot_boiler_feed_temperature_max is None and "ot_boiler_feed_temperature_max" in self.model_fields_set:
            _dict['otBoilerFeedTemperatureMax'] = None

        # set to None if ot_boiler_return_temperature_average (nullable) is None
        # and model_fields_set contains the field
        if self.ot_boiler_return_temperature_average is None and "ot_boiler_return_temperature_average" in self.model_fields_set:
            _dict['otBoilerReturnTemperatureAverage'] = None

        # set to None if ot_boiler_return_temperature_min (nullable) is None
        # and model_fields_set contains the field
        if self.ot_boiler_return_temperature_min is None and "ot_boiler_return_temperature_min" in self.model_fields_set:
            _dict['otBoilerReturnTemperatureMin'] = None

        # set to None if ot_boiler_return_temperature_max (nullable) is None
        # and model_fields_set contains the field
        if self.ot_boiler_return_temperature_max is None and "ot_boiler_return_temperature_max" in self.model_fields_set:
            _dict['otBoilerReturnTemperatureMax'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of HeatPumpLogViewDto from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "timeBucket": obj.get("timeBucket"),
            "interval": obj.get("interval"),
            "timeCoveredInInterval": obj.get("timeCoveredInInterval"),
            "heatPumpStateStandby": obj.get("heatPumpStateStandby"),
            "heatPumpStateHeating": obj.get("heatPumpStateHeating"),
            "heatPumpStateDefrost": obj.get("heatPumpStateDefrost"),
            "heatPumpStateCooling": obj.get("heatPumpStateCooling"),
            "heatPumpStateDHW": obj.get("heatPumpStateDHW"),
            "heatPumpStateLegionella": obj.get("heatPumpStateLegionella"),
            "rpmControlStartup": obj.get("rpmControlStartup"),
            "rpmControlRunning": obj.get("rpmControlRunning"),
            "rpmControlRunningAbnormally": obj.get("rpmControlRunningAbnormally"),
            "rpmControlShutdown": obj.get("rpmControlShutdown"),
            "controlBridgeStatusWaterPump": obj.get("controlBridgeStatusWaterPump"),
            "controlBridgeStatusDhwValve": obj.get("controlBridgeStatusDhwValve"),
            "controlBridgeStatusGasBoiler": obj.get("controlBridgeStatusGasBoiler"),
            "controlBridgeStatusElectricHeater": obj.get("controlBridgeStatusElectricHeater"),
            "controlBridgeStatusWaterPump2": obj.get("controlBridgeStatusWaterPump2"),
            "dtcNone": obj.get("dtcNone"),
            "dtcContinue": obj.get("dtcContinue"),
            "dtcCompressorOff": obj.get("dtcCompressorOff"),
            "dtcDefrostForbidden": obj.get("dtcDefrostForbidden"),
            "dtcRequestService": obj.get("dtcRequestService"),
            "dtcUseHeatingCurve": obj.get("dtcUseHeatingCurve"),
            "dtcDhwForbidden": obj.get("dtcDhwForbidden"),
            "dtcError": obj.get("dtcError"),
            "dtcInactive": obj.get("dtcInactive"),
            "signalStrengthAverage": obj.get("signalStrengthAverage"),
            "signalStrengthMin": obj.get("signalStrengthMin"),
            "signalStrengthMax": obj.get("signalStrengthMax"),
            "t1Average": obj.get("t1Average"),
            "t1Min": obj.get("t1Min"),
            "t1Max": obj.get("t1Max"),
            "t2Average": obj.get("t2Average"),
            "t2Min": obj.get("t2Min"),
            "t2Max": obj.get("t2Max"),
            "tSpareNtcAverage": obj.get("tSpareNtcAverage"),
            "tSpareNtcMin": obj.get("tSpareNtcMin"),
            "tSpareNtcMax": obj.get("tSpareNtcMax"),
            "tBoardAverage": obj.get("tBoardAverage"),
            "tBoardMin": obj.get("tBoardMin"),
            "tBoardMax": obj.get("tBoardMax"),
            "tAirInAverage": obj.get("tAirInAverage"),
            "tAirInMin": obj.get("tAirInMin"),
            "tAirInMax": obj.get("tAirInMax"),
            "tAirOutAverage": obj.get("tAirOutAverage"),
            "tAirOutMin": obj.get("tAirOutMin"),
            "tAirOutMax": obj.get("tAirOutMax"),
            "tWaterInAverage": obj.get("tWaterInAverage"),
            "tWaterInMin": obj.get("tWaterInMin"),
            "tWaterInMax": obj.get("tWaterInMax"),
            "tWaterOutAverage": obj.get("tWaterOutAverage"),
            "tWaterOutMin": obj.get("tWaterOutMin"),
            "tWaterOutMax": obj.get("tWaterOutMax"),
            "tWaterHouseInAverage": obj.get("tWaterHouseInAverage"),
            "tWaterHouseInMin": obj.get("tWaterHouseInMin"),
            "tWaterHouseInMax": obj.get("tWaterHouseInMax"),
            "rpmAverage": obj.get("rpmAverage"),
            "rpmMin": obj.get("rpmMin"),
            "rpmMax": obj.get("rpmMax"),
            "rpmLimiterAverage": obj.get("rpmLimiterAverage"),
            "rpmLimiterMin": obj.get("rpmLimiterMin"),
            "rpmLimiterMax": obj.get("rpmLimiterMax"),
            "rpmLimiterNoLimit": obj.get("rpmLimiterNoLimit"),
            "rpmLimiterPowerLimit": obj.get("rpmLimiterPowerLimit"),
            "rpmLimiterDefrost": obj.get("rpmLimiterDefrost"),
            "rpmLimiterSilentHours": obj.get("rpmLimiterSilentHours"),
            "rpmLimiterHPControl": obj.get("rpmLimiterHPControl"),
            "rpmLimiterPressure": obj.get("rpmLimiterPressure"),
            "rpmLimiterWaterOut": obj.get("rpmLimiterWaterOut"),
            "rpmLimiterEnvelope": obj.get("rpmLimiterEnvelope"),
            "rpmLimiterHouseIn": obj.get("rpmLimiterHouseIn"),
            "pCompressorInAverage": obj.get("pCompressorInAverage"),
            "pCompressorInMin": obj.get("pCompressorInMin"),
            "pCompressorInMax": obj.get("pCompressorInMax"),
            "pCompressorOutAverage": obj.get("pCompressorOutAverage"),
            "pCompressorOutMin": obj.get("pCompressorOutMin"),
            "pCompressorOutMax": obj.get("pCompressorOutMax"),
            "pCompressorInTargetAverage": obj.get("pCompressorInTargetAverage"),
            "pCompressorInTargetMin": obj.get("pCompressorInTargetMin"),
            "pCompressorInTargetMax": obj.get("pCompressorInTargetMax"),
            "tInverterAverage": obj.get("tInverterAverage"),
            "tInverterMin": obj.get("tInverterMin"),
            "tInverterMax": obj.get("tInverterMax"),
            "tCompressorInAverage": obj.get("tCompressorInAverage"),
            "tCompressorInMin": obj.get("tCompressorInMin"),
            "tCompressorInMax": obj.get("tCompressorInMax"),
            "tCompressorOutAverage": obj.get("tCompressorOutAverage"),
            "tCompressorOutMin": obj.get("tCompressorOutMin"),
            "tCompressorOutMax": obj.get("tCompressorOutMax"),
            "tCompressorInTransientAverage": obj.get("tCompressorInTransientAverage"),
            "tCompressorInTransientMin": obj.get("tCompressorInTransientMin"),
            "tCompressorInTransientMax": obj.get("tCompressorInTransientMax"),
            "tCompressorOutTransientAverage": obj.get("tCompressorOutTransientAverage"),
            "tCompressorOutTransientMin": obj.get("tCompressorOutTransientMin"),
            "tCompressorOutTransientMax": obj.get("tCompressorOutTransientMax"),
            "deltaTCompressorInSuperheatAverage": obj.get("deltaTCompressorInSuperheatAverage"),
            "deltaTCompressorInSuperheatMin": obj.get("deltaTCompressorInSuperheatMin"),
            "deltaTCompressorInSuperheatMax": obj.get("deltaTCompressorInSuperheatMax"),
            "compressorPowerLowAccuracyAverage": obj.get("compressorPowerLowAccuracyAverage"),
            "compressorPowerLowAccuracyMin": obj.get("compressorPowerLowAccuracyMin"),
            "compressorPowerLowAccuracyMax": obj.get("compressorPowerLowAccuracyMax"),
            "fanAverage": obj.get("fanAverage"),
            "fanMin": obj.get("fanMin"),
            "fanMax": obj.get("fanMax"),
            "fanPowerAverage": obj.get("fanPowerAverage"),
            "fanPowerMin": obj.get("fanPowerMin"),
            "fanPowerMax": obj.get("fanPowerMax"),
            "cmMassPowerInAverage": obj.get("cmMassPowerInAverage"),
            "cmMassPowerOutAverage": obj.get("cmMassPowerOutAverage"),
            "temperatureErrorIntegralAverage": obj.get("temperatureErrorIntegralAverage"),
            "temperatureErrorIntegralMin": obj.get("temperatureErrorIntegralMin"),
            "temperatureErrorIntegralMax": obj.get("temperatureErrorIntegralMax"),
            "thermostatStateOff": obj.get("thermostatStateOff"),
            "thermostatStateOn": obj.get("thermostatStateOn"),
            "tRoomAverage": obj.get("tRoomAverage"),
            "tRoomMin": obj.get("tRoomMin"),
            "tRoomMax": obj.get("tRoomMax"),
            "tRoomTargetAverage": obj.get("tRoomTargetAverage"),
            "tRoomTargetMin": obj.get("tRoomTargetMin"),
            "tRoomTargetMax": obj.get("tRoomTargetMax"),
            "tThermostatSetpointAverage": obj.get("tThermostatSetpointAverage"),
            "tThermostatSetpointMin": obj.get("tThermostatSetpointMin"),
            "tThermostatSetpointMax": obj.get("tThermostatSetpointMax"),
            "otBoilerFeedTemperatureAverage": obj.get("otBoilerFeedTemperatureAverage"),
            "otBoilerFeedTemperatureMin": obj.get("otBoilerFeedTemperatureMin"),
            "otBoilerFeedTemperatureMax": obj.get("otBoilerFeedTemperatureMax"),
            "otBoilerReturnTemperatureAverage": obj.get("otBoilerReturnTemperatureAverage"),
            "otBoilerReturnTemperatureMin": obj.get("otBoilerReturnTemperatureMin"),
            "otBoilerReturnTemperatureMax": obj.get("otBoilerReturnTemperatureMax")
        })
        return _obj


